@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -80,10 +80,13 @@ extern "C"
   /// \param min the minor version number of the library.
   ///
   /// \param rev the revision version number of the library.
+  ///
+  /// \param suf the version suffix of the library.
   void
   abigail_get_library_version(std::string& maj,
 			      std::string& min,
-			      std::string& rev);
+			      std::string& rev,
+			      std::string& suf);
 }
 
 }//end namespace abigail
