@@ -1122,7 +1122,7 @@ is_match_point(RandomAccessOutputIterator a_begin,
 /// section 4b in the paper.  As the paper says, "The idea for doing
 /// so is to simultaneously run the basic algorithm in both the
 /// forward and reverse directions until furthest reaching forward and
-/// reverse paths starting at opposing corners ‘‘overlap’’."
+/// reverse paths starting at opposing corners 'overlap'."
 ///
 /// @tparm RandomAccessOutputIterator the type of iterators passed to
 /// this function.  It must be a random access output iterator kind.
@@ -1591,8 +1591,8 @@ compute_diff(RandomAccessOutputIterator a_base,
       point px, pu;
       snake_end_points(snak, px, pu);
       compute_diff<RandomAccessOutputIterator,
-		   EqualityFunctor>(a_base, a_begin, a_base + px.x() + 1,
-				    b_base, b_begin, b_base + px.y() + 1,
+		   EqualityFunctor>(a_base, a_begin, a_base + (px.x() + 1),
+				    b_base, b_begin, b_base + (px.y() + 1),
 				    lcs, tmp_ses0, tmp_ses_len0);
 
       lcs.insert(lcs.end(), trace.begin(), trace.end());
@@ -1600,8 +1600,8 @@ compute_diff(RandomAccessOutputIterator a_base,
       int tmp_ses_len1 = 0;
       edit_script tmp_ses1;
       compute_diff<RandomAccessOutputIterator,
-		   EqualityFunctor>(a_base, a_base + pu.x() + 1, a_end,
-				    b_base, b_base + pu.y() + 1, b_end,
+		   EqualityFunctor>(a_base, a_base + (pu.x() + 1), a_end,
+				    b_base, b_base + (pu.y() + 1), b_end,
 				    lcs, tmp_ses1, tmp_ses_len1);
       ABG_ASSERT(tmp_ses0.length() + tmp_ses1.length() == d);
       ABG_ASSERT(tmp_ses_len0 + tmp_ses_len1 == d);
@@ -2046,7 +2046,7 @@ display_edit_script(const edit_script& es,
   else if (es.num_deletions() == 1)
     {
       out << "1 deletion:\n"
-	  << "\t happened at index: ";;
+	  << "\t happened at index: ";
     }
   else
     {
