@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2019 Red Hat, Inc.
+// Copyright (C) 2013-2020 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -103,6 +103,9 @@ void
 add_read_context_suppressions(read_context& ctxt,
 			      const suppr::suppressions_type& supprs);
 
+void
+consider_types_not_reachable_from_public_interfaces(read_context& ctxt,
+						    bool flag);
 }//end xml_reader
 }//end namespace abigail
 
