@@ -38,6 +38,13 @@ namespace xml_writer
 
 using namespace abigail::ir;
 
+/// The style of type id the XML writer will output.
+enum type_id_style_kind
+{
+  SEQUENCE_TYPE_ID_STYLE,
+  HASH_TYPE_ID_STYLE
+};
+
 class write_context;
 
 /// A convenience typedef for a shared pointer to write_context.
@@ -62,9 +69,21 @@ set_write_corpus_path(write_context& ctxt, bool flag);
 void
 set_write_comp_dir(write_context& ctxt, bool flag);
 
+void
+set_write_elf_needed(write_context& ctxt, bool flag);
+
+void
+set_write_default_sizes(write_context& ctxt, bool flag);
+
 void
 set_short_locs(write_context& ctxt, bool flag);
 
+void
+set_write_parameter_names(write_context& ctxt, bool flag);
+
+void
+set_type_id_style(write_context& ctxt, type_id_style_kind style);
+
 /// A convenience generic function to set common options (usually used
 /// by Libabigail tools) from a generic options carrying-object, into
 /// a given @ref write_context.
@@ -83,7 +102,11 @@ set_common_options(write_context& ctxt, const OPTS& opts)
   set_write_architecture(ctxt, opts.write_architecture);
   set_write_corpus_path(ctxt, opts.write_corpus_path);
   set_write_comp_dir(ctxt, opts.write_comp_dir);
+  set_write_elf_needed(ctxt, opts.write_elf_needed);
+  set_write_parameter_names(ctxt, opts.write_parameter_names);
   set_short_locs(ctxt, opts.short_locs);
+  set_write_default_sizes(ctxt, opts.default_sizes);
+  set_type_id_style(ctxt, opts.type_id_style);
 }
 
 void
