@@ -191,9 +191,6 @@ struct diff_context::priv
   vector<filtering::filter_base_sptr>	filters_;
   suppressions_type			suppressions_;
   pointer_map				visited_diff_nodes_;
-  // This is the last visited diff node, per class of equivalence.
-  // It's set during the redundant diff node marking process.
-  pointer_map				last_visited_diff_node_;
   corpus_diff_sptr			corpus_diff_;
   ostream*				default_output_stream_;
   ostream*				error_output_stream_;
@@ -305,11 +302,7 @@ public:
   /// @returnt a smart pointer to the diff context.
   diff_context_sptr
   get_context() const
-  {
-    if (ctxt_.expired())
-      return diff_context_sptr();
-    return diff_context_sptr(ctxt_);
-  }
+  {return ctxt_.lock();}
 
   /// Check if a given categorization of a diff node should make it be
   /// filtered out.
@@ -502,8 +495,17 @@ struct class_or_union_diff::priv
   // member bar at offset N.
   unsigned_var_diff_sptr_map changed_dm_;
   var_diff_sptrs_type sorted_changed_dm_;
+
+  // This is a data structure to represent data members that have been
+  // replaced by anonymous data members.  It's a map that associates
+  // the name of the data member to the anonymous data member that
+  // replaced it.
+  string_decl_base_sptr_map dms_replaced_by_adms_;
+  mutable changed_var_sptrs_type dms_replaced_by_adms_ordered_;
   string_member_function_sptr_map deleted_member_functions_;
+  class_or_union::member_functions sorted_deleted_member_functions_;
   string_member_function_sptr_map inserted_member_functions_;
+  class_or_union::member_functions sorted_inserted_member_functions_;
   string_function_decl_diff_sptr_map changed_member_functions_;
   function_decl_diff_sptrs_type sorted_changed_member_functions_;
   string_decl_base_sptr_map deleted_member_class_tmpls_;
@@ -549,18 +551,18 @@ struct class_or_union_diff::priv
 /// offset.
 struct data_member_comp
 {
-  /// @param f the first data member to take into account.
-  ///
-  /// @param s the second data member to take into account.
-  ///
-  /// @return true iff f is before s.
-  bool
-  operator()(const decl_base_sptr& f,
-	     const decl_base_sptr& s) const
-  {
-    var_decl_sptr first_dm = is_data_member(f);
-    var_decl_sptr second_dm = is_data_member(s);
 
+  /// Compare two data members.
+  ///
+  /// First look at their offset and then their name.
+  ///
+  /// @parm first_dm the first data member to consider.
+  ///
+  /// @param second_dm the second data member to consider.
+  bool
+  compare_data_members(const var_decl_sptr& first_dm,
+		       const var_decl_sptr& second_dm) const
+  {
     ABG_ASSERT(first_dm);
     ABG_ASSERT(second_dm);
 
@@ -578,6 +580,40 @@ struct data_member_comp
     // sort them lexicographically.
     return first_dm_name < second_dm_name;
   }
+
+  /// Compare two data members.
+  ///
+  /// First look at their offset and then their name.
+  ///
+  /// @parm first_dm the first data member to consider.
+  ///
+  /// @param second_dm the second data member to consider.
+  bool
+  operator()(const decl_base_sptr& f,
+	     const decl_base_sptr& s) const
+  {
+    var_decl_sptr first_dm = is_data_member(f);
+    var_decl_sptr second_dm = is_data_member(s);
+
+    return compare_data_members(first_dm, second_dm);
+  }
+
+  /// Compare two data members.
+  ///
+  /// First look at their offset and then their name.
+  ///
+  /// @parm first_dm the first data member to consider.
+  ///
+  /// @param second_dm the second data member to consider.
+  bool
+  operator()(const changed_var_sptr& f,
+	     const changed_var_sptr& s) const
+  {
+    var_decl_sptr first_dm = is_data_member(is_decl(f.first));
+    var_decl_sptr second_dm = is_data_member(is_decl(s.first));
+
+    return compare_data_members(first_dm, second_dm);
+  }
 };//end struct data_member_comp
 
 /// The type of the private data (pimpl sub-object) of the @ref
@@ -1331,7 +1367,7 @@ struct corpus_diff::diff_stats::priv
 
   diff_context_sptr
   ctxt()
-  {return ctxt_.expired() ? diff_context_sptr() : diff_context_sptr(ctxt_);}
+  {return ctxt_.lock();}
 }; // end class corpus_diff::diff_stats::priv
 
 void
@@ -1346,13 +1382,21 @@ void
 sort_data_members(const string_decl_base_sptr_map &data_members,
 		  vector<decl_base_sptr>& sorted);
 
+void
+sort_changed_data_members(changed_var_sptrs_type& input);
+
 void
 sort_string_function_ptr_map(const string_function_ptr_map& map,
 			     vector<function_decl*>& sorted);
 
+void
+sort_string_member_function_sptr_map(const string_member_function_sptr_map& map,
+				     class_or_union::member_functions& sorted);
+
 void
 sort_string_type_base_sptr_map(string_type_base_sptr_map& map,
 			       vector<type_base_sptr>& sorted);
+
 void
 sort_string_function_decl_diff_sptr_map
 (const string_function_decl_diff_sptr_map& map,
