@@ -35,6 +35,41 @@ namespace abigail
 namespace comparison
 {
 
+/// Test if a given instance of @ref corpus_diff carries changes whose
+/// reports are not suppressed by any suppression specification.  In
+/// effect, these are deemed incompatible ABI changes.
+///
+/// @param d the @ref corpus_diff to consider
+///
+/// @return true iff @p d carries subtype changes that are deemed
+/// incompatible ABI changes.
+bool
+default_reporter::diff_has_net_changes(const corpus_diff *d) const
+{
+  if (!d)
+    return false;
+
+  const corpus_diff::diff_stats& stats = const_cast<corpus_diff*>(d)->
+    apply_filters_and_suppressions_before_reporting();
+
+  // Logic here should match emit_diff_stats.
+  return (d->architecture_changed()
+	  || d->soname_changed()
+	  || stats.net_num_func_removed()
+	  || stats.net_num_func_changed()
+	  || stats.net_num_func_added()
+	  || stats.net_num_vars_removed()
+	  || stats.net_num_vars_changed()
+	  || stats.net_num_vars_added()
+	  || stats.net_num_removed_unreachable_types()
+	  || stats.net_num_changed_unreachable_types()
+	  || stats.net_num_added_unreachable_types()
+	  || stats.net_num_removed_func_syms()
+	  || stats.net_num_added_func_syms()
+	  || stats.net_num_removed_var_syms()
+	  || stats.net_num_added_var_syms());
+}
+
 /// Ouputs a report of the differences between of the two type_decl
 /// involved in the @ref type_decl_diff.
 ///
@@ -55,33 +90,25 @@ default_reporter::report(const type_decl_diff& d,
 
   string name = f->get_pretty_representation();
 
-  bool n = report_name_size_and_alignment_changes(f, s, d.context(),
-						  out, indent,
-						  /*new line=*/false);
+  report_name_size_and_alignment_changes(f, s, d.context(),
+					 out, indent);
 
   if (f->get_visibility() != s->get_visibility())
     {
-      if (n)
-	out << "\n";
       out << indent
 	  << "visibility changed from '"
-	  << f->get_visibility() << "' to '" << s->get_visibility();
-      n = true;
+	  << f->get_visibility() << "' to '" << s->get_visibility()
+	  << "\n";
     }
 
   if (f->get_linkage_name() != s->get_linkage_name())
     {
-      if (n)
-	out << "\n";
       out << indent
 	  << "mangled name changed from '"
 	  << f->get_linkage_name() << "' to "
-	  << s->get_linkage_name();
-      n = true;
+	  << s->get_linkage_name()
+	  << "\n";
     }
-
-  if (n)
-    out << "\n";
 }
 
 /// Report the differences between the two enums.
@@ -106,11 +133,29 @@ default_reporter::report(const enum_diff& d, ostream& out,
 
   enum_type_decl_sptr first = d.first_enum(), second = d.second_enum();
 
-  if (report_name_size_and_alignment_changes(first, second, d.context(),
-					     out, indent,
-					     /*start_with_num_line=*/false))
-    out << "\n";
-  maybe_report_diff_for_member(first, second, d.context(), out, indent);
+  const diff_context_sptr& ctxt = d.context();
+
+  // Report enum decl-only <-> definition changes.
+  if (ctxt->get_allowed_category() & TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY)
+    if (filtering::has_enum_decl_only_def_change(first, second))
+      {
+	string was =
+	  first->get_is_declaration_only()
+	  ? " was a declaration-only enum type"
+	  : " was a defined enum type";
+
+	string is_now =
+	  second->get_is_declaration_only()
+	  ? " and is now a declaration-only enum type"
+	  : " and is now a defined enum type";
+
+	out << indent << "enum type " << name << was << is_now << "\n";
+	return;
+      }
+
+  report_name_size_and_alignment_changes(first, second, ctxt,
+					 out, indent);
+  maybe_report_diff_for_member(first, second, ctxt, out, indent);
 
   //underlying type
   d.underlying_type_diff()->report(out, indent);
@@ -130,16 +175,14 @@ default_reporter::report(const enum_diff& d, ostream& out,
 	   i != sorted_deleted_enumerators.end();
 	   ++i)
 	{
-	  if (i != sorted_deleted_enumerators.begin())
-	    out << "\n";
 	  out << indent
 	      << "  '"
 	      << i->get_qualified_name()
 	      << "' value '"
 	      << i->get_value()
 	      << "'";
+	  out << "\n";
 	}
-      out << "\n\n";
     }
   if (numins)
     {
@@ -151,16 +194,14 @@ default_reporter::report(const enum_diff& d, ostream& out,
 	   i != sorted_inserted_enumerators.end();
 	   ++i)
 	{
-	  if (i != sorted_inserted_enumerators.begin())
-	    out << "\n";
 	  out << indent
 	      << "  '"
 	      << i->get_qualified_name()
 	      << "' value '"
 	      << i->get_value()
 	      << "'";
+	  out << "\n";
 	}
-      out << "\n\n";
     }
   if (numchanges)
     {
@@ -173,45 +214,42 @@ default_reporter::report(const enum_diff& d, ostream& out,
 	   i != sorted_changed_enumerators.end();
 	   ++i)
 	{
-	  if (i != sorted_changed_enumerators.begin())
-	    out << "\n";
 	  out << indent
 	      << "  '"
 	      << i->first.get_qualified_name()
 	      << "' from value '"
 	      << i->first.get_value() << "' to '"
 	      << i->second.get_value() << "'";
-	  report_loc_info(second, *d.context(), out);
+	  report_loc_info(second, *ctxt, out);
+	  out << "\n";
 	}
-      out << "\n\n";
     }
 
-  if (d.context()->show_leaf_changes_only())
-    maybe_report_interfaces_impacted_by_diff(&d, out, indent,
-					     /*new_line_prefix=*/false);
+  if (ctxt->show_leaf_changes_only())
+    maybe_report_interfaces_impacted_by_diff(&d, out, indent);
 
   d.reported_once(true);
 }
 
-/// For a @ref typedef_diff node, report the changes that are local.
+/// For a @ref typedef_diff node, report the local changes to the
+/// typedef rather the changes to its underlying type.
+///
+/// Note that changes to the underlying type are also considered
+/// local.
 ///
 /// @param d the @ref typedef_diff node to consider.
 ///
 /// @param out the output stream to report to.
 ///
 /// @param indent the white space string to use for indentation.
-///
-/// @return true iff the caller needs to emit a newline to the output
-/// stream before emitting anything else.
-bool
-default_reporter::report_local_typedef_changes(const typedef_diff &d,
-					       ostream& out,
-					       const string& indent) const
+void
+default_reporter::report_non_type_typedef_changes(const typedef_diff &d,
+						  ostream& out,
+						  const string& indent) const
 {
   if (!d.to_be_reported())
-    return false;
+    return;
 
-  bool emit_nl = false;
   typedef_decl_sptr f = d.first_typedef_decl(), s = d.second_typedef_decl();
 
   maybe_report_diff_for_member(f, s, d.context(), out, indent);
@@ -224,14 +262,11 @@ default_reporter::report_local_typedef_changes(const typedef_diff &d,
     {
       out << indent << "typedef name changed from "
 	  << f->get_qualified_name()
-          << " to "
+	  << " to "
 	  << s->get_qualified_name();
       report_loc_info(s, *d.context(), out);
       out << "\n";
-      emit_nl = true;
     }
-
-  return emit_nl;
 }
 
 /// Reports the difference between the two subjects of the diff in a
@@ -252,7 +287,7 @@ default_reporter::report(const typedef_diff& d,
 
   typedef_decl_sptr f = d.first_typedef_decl(), s = d.second_typedef_decl();
 
-  bool emit_nl = report_local_typedef_changes(d, out, indent);
+  report_non_type_typedef_changes(d, out, indent);
 
   diff_sptr dif = d.underlying_type_diff();
   if (dif && dif->has_changes())
@@ -267,7 +302,6 @@ default_reporter::report(const typedef_diff& d,
 	  report_loc_info(dif->first_subject(), *d.context(), out);
 	  out << " changed:\n";
 	  dif->report(out, indent + "  ");
-	  emit_nl = false;
 	}
       else
 	{
@@ -291,14 +325,10 @@ default_reporter::report(const typedef_diff& d,
 	      dif->report(out, indent + "  ");
 	      if (c & REDUNDANT_CATEGORY)
 		dif->set_category(c | REDUNDANT_CATEGORY);
-	      emit_nl = false;
 	    }
 	}
     }
 
-  if (emit_nl)
-    out << "\n";
-
   d.reported_once(true);
 }
 
@@ -392,7 +422,7 @@ default_reporter::report(const pointer_diff& d, ostream& out,
 	: string("void");
 
       out << indent
-	  << "in pointed to type '" <<  repr << "'";
+	  << "in pointed to type '" << repr << "'";
       report_loc_info(dif->second_subject(), *d.context(), out);
       out << ":\n";
       dif->report(out, indent + "  ");
@@ -459,7 +489,7 @@ default_reporter::report(const reference_diff& d, ostream& out,
   enum change_kind k = ir::NO_CHANGE_KIND;
   equals(*d.first_reference(), *d.second_reference(), &k);
 
-  if ((k & LOCAL_CHANGE_KIND) && !(k & SUBTYPE_CHANGE_KIND))
+  if ((k & ALL_LOCAL_CHANGES_MASK) && !(k & SUBTYPE_CHANGE_KIND))
     report_local_reference_type_changes(d, out, indent);
 
   if (k & SUBTYPE_CHANGE_KIND)
@@ -506,8 +536,10 @@ default_reporter::report(const fn_parm_diff& d, ostream& out,
       diff_category saved_category = type_diff->get_category();
       // Parameter type changes are never redundants.
       type_diff->set_category(saved_category & ~REDUNDANT_CATEGORY);
-      out << indent
-	  << "parameter " << f->get_index();
+      out << indent;
+      if (f->get_is_artificial())
+	out << "implicit ";
+      out << "parameter " << f->get_index();
       report_loc_info(f, *d.context(), out);
       out << " of type '"
 	  << f->get_type_pretty_representation();
@@ -569,7 +601,6 @@ default_reporter::report_local_function_type_changes(const function_type_diff& d
   // this shouldn't be as straightforward.
 
   // Report about the parameters that got removed.
-  bool emitted = false;
   for (vector<function_decl::parameter_sptr>::const_iterator i =
 	 d.priv_->sorted_deleted_parms_.begin();
        i != d.priv_->sorted_deleted_parms_.end();
@@ -578,13 +609,9 @@ default_reporter::report_local_function_type_changes(const function_type_diff& d
       out << indent << "parameter " << (*i)->get_index()
 	  << " of type '" << (*i)->get_type_pretty_representation()
 	  << "' was removed\n";
-      emitted = true;
     }
-  if (emitted)
-    out << "\n";
 
   // Report about the parameters that got added
-  emitted = false;
   for (vector<function_decl::parameter_sptr>::const_iterator i =
 	 d.priv_->sorted_added_parms_.begin();
        i != d.priv_->sorted_added_parms_.end();
@@ -593,11 +620,7 @@ default_reporter::report_local_function_type_changes(const function_type_diff& d
       out << indent << "parameter " << (*i)->get_index()
 	  << " of type '" << (*i)->get_type_pretty_representation()
 	  << "' was added\n";
-      emitted = true;
     }
-
-  if (emitted)
-    out << "\n";
 }
 
 /// Build and emit a textual report about a @ref function_type_diff.
@@ -669,16 +692,14 @@ default_reporter::report(const array_diff& d, ostream& out,
       string fn = ir::get_pretty_representation(is_type(dif->first_subject()));
       // report array element type changes
       out << indent << "array element type '"
-	  << fn << "' changed: \n";
+	  << fn << "' changed:\n";
       dif->report(out, indent + "  ");
     }
 
   report_name_size_and_alignment_changes(d.first_array(),
 					 d.second_array(),
 					 d.context(),
-					 out, indent,
-					 /*new line=*/false);
-  report_loc_info(d.second_array(), *d.context(), out);
+					 out, indent);
 }
 
 /// Generates a report for an intance of @ref base_diff.
@@ -788,8 +809,8 @@ default_reporter::report(const scope_diff& d, ostream& out,
 
       out << indent << "  '"
 	  << (*dif)->first_subject()->get_pretty_representation()
-          << "' was changed to '"
-          << (*dif)->second_subject()->get_pretty_representation() << "'";
+	  << "' was changed to '"
+	  << (*dif)->second_subject()->get_pretty_representation() << "'";
       report_loc_info((*dif)->second_subject(), *d.context(), out);
       out << ":\n";
 
@@ -884,8 +905,8 @@ default_reporter::report(const class_or_union_diff& d,
 
   const diff_context_sptr& ctxt = d.context();
 
-  // Report class decl-only -> definition change.
-  if (ctxt->get_allowed_category() & CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY)
+  // Report class decl-only <-> definition change.
+  if (ctxt->get_allowed_category() & TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY)
     if (filtering::has_class_decl_only_def_change(first, second))
       {
 	string was =
@@ -899,7 +920,7 @@ default_reporter::report(const class_or_union_diff& d,
 	  : " and is now a defined type";
 
 	out << indent << "type " << first->get_pretty_representation()
-	    << was << is_now;
+	    << was << is_now << "\n";
 	return;
       }
 
@@ -913,27 +934,20 @@ default_reporter::report(const class_or_union_diff& d,
       if (numdels)
 	report_mem_header(out, numdels, num_filtered, del_kind,
 			  "member function", indent);
-      bool emitted = false;
-      for (string_member_function_sptr_map::const_iterator i =
-	     d.get_priv()->deleted_member_functions_.begin();
-	   i != d.get_priv()->deleted_member_functions_.end();
+      for (class_or_union::member_functions::const_iterator i =
+	     d.get_priv()->sorted_deleted_member_functions_.begin();
+	   i != d.get_priv()->sorted_deleted_member_functions_.end();
 	   ++i)
 	{
 	  if (!(ctxt->get_allowed_category()
 		& NON_VIRT_MEM_FUN_CHANGE_CATEGORY)
-	      && !get_member_function_is_virtual(i->second))
+	      && !get_member_function_is_virtual(*i))
 	    continue;
 
-	  if (emitted
-	      && i != d.get_priv()->deleted_member_functions_.begin())
-	    out << "\n";
-	  method_decl_sptr mem_fun = i->second;
+	  method_decl_sptr mem_fun = *i;
 	  out << indent << "  ";
 	  represent(*ctxt, mem_fun, out);
-	  emitted = true;
 	}
-      if (emitted)
-	out << "\n";
 
       // report insertions;
       int numins = d.get_priv()->inserted_member_functions_.size();
@@ -941,27 +955,20 @@ default_reporter::report(const class_or_union_diff& d,
       if (numins)
 	report_mem_header(out, numins, num_filtered, ins_kind,
 			  "member function", indent);
-      emitted = false;
-      for (string_member_function_sptr_map::const_iterator i =
-	     d.get_priv()->inserted_member_functions_.begin();
-	   i != d.get_priv()->inserted_member_functions_.end();
+      for (class_or_union::member_functions::const_iterator i =
+	     d.get_priv()->sorted_inserted_member_functions_.begin();
+	   i != d.get_priv()->sorted_inserted_member_functions_.end();
 	   ++i)
 	{
 	  if (!(ctxt->get_allowed_category()
 		& NON_VIRT_MEM_FUN_CHANGE_CATEGORY)
-	      && !get_member_function_is_virtual(i->second))
+	      && !get_member_function_is_virtual(*i))
 	    continue;
 
-	  if (emitted
-	      && i != d.get_priv()->inserted_member_functions_.begin())
-	    out << "\n";
-	  method_decl_sptr mem_fun = i->second;
+	  method_decl_sptr mem_fun = *i;
 	  out << indent << "  ";
 	  represent(*ctxt, mem_fun, out);
-	  emitted = true;
 	}
-      if (emitted)
-	out << "\n";
 
       // report member function with sub-types changes
       int numchanges = d.get_priv()->sorted_changed_member_functions_.size();
@@ -969,7 +976,6 @@ default_reporter::report(const class_or_union_diff& d,
       if (numchanges)
 	report_mem_header(out, numchanges, num_filtered, change_kind,
 			  "member function", indent);
-      emitted = false;
       for (function_decl_diff_sptrs_type::const_iterator i =
 	     d.get_priv()->sorted_changed_member_functions_.begin();
 	   i != d.get_priv()->sorted_changed_member_functions_.end();
@@ -989,15 +995,9 @@ default_reporter::report(const class_or_union_diff& d,
 
 	  string repr =
 	    (*i)->first_function_decl()->get_pretty_representation();
-	  if (emitted
-	      && i != d.get_priv()->sorted_changed_member_functions_.begin())
-	    out << "\n";
 	  out << indent << "  '" << repr << "' has some sub-type changes:\n";
 	  diff->report(out, indent + "    ");
-	  emitted = true;
 	}
-      if (numchanges)
-	out << "\n";
     }
 
   // data members
@@ -1014,7 +1014,6 @@ default_reporter::report(const class_or_union_diff& d,
 	  sort_data_members
 	    (d.class_or_union_diff::get_priv()->deleted_data_members_,
 	     sorted_dms);
-	  bool emitted = false;
 	  for (vector<decl_base_sptr>::const_iterator i = sorted_dms.begin();
 	       i != sorted_dms.end();
 	       ++i)
@@ -1024,14 +1023,8 @@ default_reporter::report(const class_or_union_diff& d,
 	      ABG_ASSERT(data_mem);
 	      if (get_member_is_static(data_mem))
 		continue;
-	      if (emitted)
-		out << "\n";
-	      out << indent << "  ";
-	      represent_data_member(data_mem, ctxt, out);
-	      emitted = true;
+	      represent_data_member(data_mem, ctxt, out, indent + "  ");
 	    }
-	  if (emitted)
-	    out << "\n";
 	}
 
       //report insertions
@@ -1052,52 +1045,42 @@ default_reporter::report(const class_or_union_diff& d,
 	      var_decl_sptr data_mem =
 		dynamic_pointer_cast<var_decl>(*i);
 	      ABG_ASSERT(data_mem);
-	      out << indent << "  ";
-	      represent_data_member(data_mem, ctxt, out);
+	      represent_data_member(data_mem, ctxt, out, indent + "  ");
 	    }
 	}
 
       // report change
-      size_t numchanges =
-	d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.size();
-      size_t num_filtered =
-	d.class_or_union_diff::get_priv()->count_filtered_subtype_changed_dm();
-      if (numchanges)
+      size_t num_changes =
+	(d.sorted_subtype_changed_data_members().size()
+	 + d.sorted_changed_data_members().size());
+
+      size_t num_changes_filtered =
+	(d.count_filtered_subtype_changed_data_members()
+	 + d.count_filtered_changed_data_members());
+
+      if (num_changes)
 	{
-	  report_mem_header(out, numchanges, num_filtered,
-			    subtype_change_kind, "data member", indent);
+	  report_mem_header(out, num_changes, num_changes_filtered,
+			    change_kind, "data member", indent);
+
 	  for (var_diff_sptrs_type::const_iterator it =
-		 d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.begin();
-	       it != d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.end();
+		 d.sorted_changed_data_members().begin();
+	       it != d.sorted_changed_data_members().end();
 	       ++it)
-	    {
-	      if ((*it)->to_be_reported())
-		{
-		  represent(*it, ctxt, out, indent + " ");
-		  out << "\n";
-		}
-	    }
+	    if ((*it)->to_be_reported())
+	      represent(*it, ctxt, out, indent + "  ");
+
+	  for (var_diff_sptrs_type::const_iterator it =
+		 d.sorted_subtype_changed_data_members().begin();
+	       it != d.sorted_subtype_changed_data_members().end();
+	       ++it)
+	    if ((*it)->to_be_reported())
+	      represent(*it, ctxt, out, indent + "  ");
 	}
 
-      numchanges = d.class_or_union_diff::get_priv()->sorted_changed_dm_.size();
-      num_filtered =
-	d.class_or_union_diff::get_priv()->count_filtered_changed_dm();
-      if (numchanges)
-	{
-	  report_mem_header(out, numchanges, num_filtered,
-			    change_kind, "data member", indent);
-	  for (var_diff_sptrs_type::const_iterator it =
-		 d.class_or_union_diff::get_priv()->sorted_changed_dm_.begin();
-	       it != d.class_or_union_diff::get_priv()->sorted_changed_dm_.end();
-	       ++it)
-	    {
-	      if ((*it)->to_be_reported())
-		{
-		  represent(*it, ctxt, out, indent + " ");
-		  out << "\n";
-		}
-	    }
-	}
+      // Report about data members replaced by an anonymous union data
+      // member.
+      maybe_report_data_members_replaced_by_anon_dm(d, out, indent);
     }
 
   // member types
@@ -1119,14 +1102,12 @@ default_reporter::report(const class_or_union_diff& d,
 	       i != d.class_or_union_diff::get_priv()->deleted_member_types_.end();
 	       ++i)
 	    {
-	      if (i != d.class_or_union_diff::get_priv()->deleted_member_types_.begin())
-		out << "\n";
 	      decl_base_sptr mem_type = i->second;
 	      out << indent << "  '"
 		  << mem_type->get_pretty_representation()
-		  << "'";
+		  << "'\n";
 	    }
-	  out << "\n\n";
+	  out << "\n";
 	}
       // report changes
       if (numchanges)
@@ -1164,7 +1145,6 @@ default_reporter::report(const class_or_union_diff& d,
 	  report_mem_header(out, numins, 0, ins_kind,
 			    "member type", indent);
 
-	  bool emitted = false;
 	  for (vector<insertion>::const_iterator i = e.insertions().begin();
 	       i != e.insertions().end();
 	       ++i)
@@ -1175,8 +1155,6 @@ default_reporter::report(const class_or_union_diff& d,
 		   j != i->inserted_indexes().end();
 		   ++j)
 		{
-		  if (emitted)
-		    out << "\n";
 		  mem_type = second->get_member_types()[*j];
 		  if (!d.class_or_union_diff::get_priv()->
 		      member_type_has_changed(get_type_declaration(mem_type)))
@@ -1184,12 +1162,11 @@ default_reporter::report(const class_or_union_diff& d,
 		      out << indent << "  '"
 			  << get_type_declaration(mem_type)->
 			get_pretty_representation()
-			  << "'";
-		      emitted = true;
+			  << "'\n";
 		    }
 		}
 	    }
-	  out << "\n\n";
+	  out << "\n";
 	}
     }
 
@@ -1205,23 +1182,18 @@ default_reporter::report(const class_or_union_diff& d,
 	   i != e.deletions().end();
 	   ++i)
 	{
-	  if (i != e.deletions().begin())
-	    out << "\n";
 	  member_function_template_sptr mem_fn_tmpl =
 	    first->get_member_function_templates()[i->index()];
 	  out << indent << "  '"
 	      << mem_fn_tmpl->as_function_tdecl()->get_pretty_representation()
-	      << "'";
+	      << "'\n";
 	}
-      if (numdels)
-	out << "\n\n";
 
       // report insertions
       int numins = e.num_insertions();
       if (numins)
 	report_mem_header(out, numins, 0, ins_kind,
 			  "member function template", indent);
-      bool emitted = false;
       for (vector<insertion>::const_iterator i = e.insertions().begin();
 	   i != e.insertions().end();
 	   ++i)
@@ -1232,18 +1204,13 @@ default_reporter::report(const class_or_union_diff& d,
 	       j != i->inserted_indexes().end();
 	       ++j)
 	    {
-	      if (emitted)
-		out << "\n";
 	      mem_fn_tmpl = second->get_member_function_templates()[*j];
 	      out << indent << "  '"
 		  << mem_fn_tmpl->as_function_tdecl()->
 		get_pretty_representation()
-		  << "'";
-	      emitted = true;
+		  << "'\n";
 	    }
 	}
-      if (numins)
-	out << "\n\n";
     }
 
   // member class templates.
@@ -1258,23 +1225,18 @@ default_reporter::report(const class_or_union_diff& d,
 	   i != e.deletions().end();
 	   ++i)
 	{
-	  if (i != e.deletions().begin())
-	    out << "\n";
 	  member_class_template_sptr mem_cls_tmpl =
 	    first->get_member_class_templates()[i->index()];
 	  out << indent << "  '"
 	      << mem_cls_tmpl->as_class_tdecl()->get_pretty_representation()
-	      << "'";
+	      << "'\n";
 	}
-      if (numdels)
-	out << "\n\n";
 
       // report insertions
       int numins = e.num_insertions();
       if (numins)
 	report_mem_header(out, numins, 0, ins_kind,
 			  "member class template", indent);
-      bool emitted = false;
       for (vector<insertion>::const_iterator i = e.insertions().begin();
 	   i != e.insertions().end();
 	   ++i)
@@ -1285,18 +1247,13 @@ default_reporter::report(const class_or_union_diff& d,
 	       j != i->inserted_indexes().end();
 	       ++j)
 	    {
-	      if (emitted)
-		out << "\n";
 	      mem_cls_tmpl = second->get_member_class_templates()[*j];
 	      out << indent << "  '"
 		  << mem_cls_tmpl->as_class_tdecl()
 		->get_pretty_representation()
-		  << "'";
-	      emitted = true;
+		  << "'\n";
 	    }
 	}
-      if (numins)
-	out << "\n\n";
     }
 }
 
@@ -1327,10 +1284,8 @@ default_reporter::report(const class_diff& d, ostream& out,
   class_decl_sptr first = d.first_class_decl(),
     second = d.second_class_decl();
 
-  if (report_name_size_and_alignment_changes(first, second, d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second, d.context(),
+					 out, indent);
 
   const diff_context_sptr& ctxt = d.context();
   maybe_report_diff_for_member(first, second, ctxt, out, indent);
@@ -1367,7 +1322,6 @@ default_reporter::report(const class_diff& d, ostream& out,
 	}
 
       // Report changes.
-      bool emitted = false;
       size_t num_filtered = d.get_priv()->count_filtered_bases();
       if (numchanges)
 	{
@@ -1388,10 +1342,7 @@ default_reporter::report(const class_diff& d, ostream& out,
 	      report_loc_info(o->get_base_class(), *d.context(), out);
 	      out << " changed:\n";
 	      diff->report(out, indent + "    ");
-	      emitted = true;
 	    }
-	  if (emitted)
-	    out << "\n";
 	}
 
       //Report insertions.
@@ -1401,20 +1352,16 @@ default_reporter::report(const class_diff& d, ostream& out,
 	  report_mem_header(out, numins, 0, ins_kind,
 			    "base class", indent);
 
-	  bool emitted = false;
 	  for (class_decl::base_specs::const_iterator i =
 		 d.get_priv()->sorted_inserted_bases_.begin();
 	       i != d.get_priv()->sorted_inserted_bases_.end();
 	       ++i)
 	    {
 	      class_decl_sptr b = (*i)->get_base_class();
-	      if (emitted)
-		out << "\n";
 	      out << indent << "  " << b->get_pretty_representation();
 	      report_loc_info(b, *ctxt, out);
-	      emitted = true;
+	      out << "\n";
 	    }
-	  out << "\n";
 	}
     }
 
@@ -1446,10 +1393,8 @@ default_reporter::report(const union_diff& d, ostream& out,
   // Now report the changes about the differents parts of the type.
   union_decl_sptr first = d.first_union_decl(), second = d.second_union_decl();
 
-  if (report_name_size_and_alignment_changes(first, second, d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second, d.context(),
+					 out, indent);
 
   maybe_report_diff_for_member(first, second,d. context(), out, indent);
 
@@ -1464,15 +1409,18 @@ default_reporter::report(const union_diff& d, ostream& out,
       out << indent << "type changed from:\n"
 	  << get_class_or_union_flat_representation(first, indent + "  ",
 						    /*one_line=*/true,
+						    /*internal=*/false,
 						    /*qualified_names=*/false)
 	  << "\n"
 	  << indent << "to:\n"
 	  << get_class_or_union_flat_representation(second, indent + "  ",
 						    /*one_line=*/true,
-						    /*qualified_names=*/false);
+						    /*internal=*/false,
+						    /*qualified_names=*/false)
+	  << "\n";
     }
 
- d.currently_reporting(false);
+  d.currently_reporting(false);
 
   d.reported_once(true);
 }
@@ -1512,9 +1460,7 @@ default_reporter::report(const distinct_diff& d, ostream& out,
   if (diff)
     diff->report(out, indent + "  ");
   else
-    if (report_size_and_alignment_changes(f, s, d.context(), out, indent,
-					  /*start_with_new_line=*/false))
-      out << "\n";
+    report_size_and_alignment_changes(f, s, d.context(), out, indent);
 }
 
 /// Serialize a report of the changes encapsulated in the current
@@ -1710,11 +1656,9 @@ default_reporter::report(const var_diff& d, ostream& out,
   decl_base_sptr first = d.first_var(), second = d.second_var();
   string n = first->get_pretty_representation();
 
-  if (report_name_size_and_alignment_changes(first, second,
-					     d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second,
+					 d.context(),
+					 out, indent);
 
   maybe_report_diff_for_symbol(d.first_var()->get_symbol(),
 			       d.second_var()->get_symbol(),
@@ -1728,7 +1672,7 @@ default_reporter::report(const var_diff& d, ostream& out,
 	{
 	  RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER2(dif, "type");
 	  out << indent << "type of variable changed:\n";
-	  dif->report(out, indent + " ");
+	  dif->report(out, indent + "  ");
 	}
     }
 }
@@ -1761,18 +1705,12 @@ void
 default_reporter::report(const corpus_diff& d, ostream& out,
 			 const string& indent) const
 {
-  size_t total = 0, removed = 0, added = 0;
   const corpus_diff::diff_stats &s =
     const_cast<corpus_diff&>(d).
     apply_filters_and_suppressions_before_reporting();
 
   const diff_context_sptr& ctxt = d.context();
 
-  /// Report removed/added/changed functions.
-  total = s.net_num_func_removed() + s.net_num_func_added() +
-    s.net_num_func_changed();
-  const unsigned large_num = 100;
-
   d.priv_->emit_diff_stats(s, out, indent);
   if (ctxt->show_stats_only())
     return;
@@ -1790,6 +1728,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	<< d.first_corpus()->get_architecture_name() << "' to '"
 	<< d.second_corpus()->get_architecture_name() << "'\n\n";
 
+  /// Report removed/added/changed functions.
   if (ctxt->show_deleted_fns())
     {
       if (s.net_num_func_removed() == 1)
@@ -1797,6 +1736,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
       else if (s.net_num_func_removed() > 1)
 	out << indent << s.net_num_func_removed() << " Removed functions:\n\n";
 
+      bool emitted = false;
       vector<function_decl*>sorted_deleted_fns;
       sort_string_function_ptr_map(d.priv_->deleted_fns_, sorted_deleted_fns);
       for (vector<function_decl*>::const_iterator i =
@@ -1809,8 +1749,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 
 	  out << indent
 	      << "  ";
-	  if (total > large_num)
-	    out << "[D] ";
+	  out << "[D] ";
 	  out << "'" << (*i)->get_pretty_representation() << "'";
 	  if (ctxt->show_linkage_names())
 	    {
@@ -1830,9 +1769,9 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 		  << c->get_pretty_representation()
 		  << "\n";
 	    }
-	  ++removed;
+	  emitted = true;
 	}
-      if (removed)
+      if (emitted)
 	out << "\n";
     }
 
@@ -1843,6 +1782,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
       else if (s.net_num_func_added() > 1)
 	out << indent << s.net_num_func_added()
 	    << " Added functions:\n\n";
+      bool emitted = false;
       vector<function_decl*> sorted_added_fns;
       sort_string_function_ptr_map(d.priv_->added_fns_, sorted_added_fns);
       for (vector<function_decl*>::const_iterator i = sorted_added_fns.begin();
@@ -1855,8 +1795,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	  out
 	    << indent
 	    << "  ";
-	  if (total > large_num)
-	    out << "[A] ";
+	  out << "[A] ";
 	  out << "'"
 	      << (*i)->get_pretty_representation()
 	      << "'";
@@ -1879,13 +1818,10 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 		  << c->get_pretty_representation()
 		  << "\n";
 	    }
-	  ++added;
-	}
-      if (added)
-	{
-	  out << "\n";
-	  added = false;
+	  emitted = true;
 	}
+      if (emitted)
+	out << "\n";
     }
 
   if (ctxt->show_changed_fns())
@@ -1897,7 +1833,6 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	out << indent << num_changed
 	    << " functions with some indirect sub-type change:\n\n";
 
-      bool emitted = false;
       vector<function_decl_diff_sptr> sorted_changed_fns;
       sort_string_function_decl_diff_sptr_map(d.priv_->changed_fns_map_,
 					      sorted_changed_fns);
@@ -1913,7 +1848,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	  if (diff->to_be_reported())
 	    {
 	      function_decl_sptr fn = (*i)->first_function_decl();
-	      out << indent << "  [C]'"
+	      out << indent << "  [C] '"
 		  << fn->get_pretty_representation() << "'";
 	      report_loc_info((*i)->second_function_decl(), *ctxt, out);
 	      out << " has some indirect sub-type changes:\n";
@@ -1962,21 +1897,14 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 		    }
 		}
 	      diff->report(out, indent + "    ");
+	      // Extra spacing.
 	      out << "\n";
-	      emitted |= true;
 	    }
 	}
-      if (emitted)
-	{
-	  out << "\n";
-	  emitted = false;
-	}
+      // Changed functions have extra spacing already. No new line here.
     }
 
-  // Report added/removed/changed variables.
-  total = s.num_vars_removed() + s.num_vars_added() +
-    s.num_vars_changed() - s.num_changed_vars_filtered_out();
-
+  // Report removed/added/changed variables.
   if (ctxt->show_deleted_vars())
     {
       if (s.net_num_vars_removed() == 1)
@@ -1985,6 +1913,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	out << indent << s.net_num_vars_removed()
 	    << " Removed variables:\n\n";
       string n;
+      bool emitted = false;
       vector<var_decl*> sorted_deleted_vars;
       sort_string_var_ptr_map(d.priv_->deleted_vars_, sorted_deleted_vars);
       for (vector<var_decl*>::const_iterator i =
@@ -1999,8 +1928,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 
 	  out << indent
 	      << "  ";
-	  if (total > large_num)
-	    out << "[D] ";
+	  out << "[D] ";
 	  out << "'"
 	      << n
 	      << "'";
@@ -2012,13 +1940,10 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	      out << "}";
 	    }
 	  out << "\n";
-	  ++removed;
+	  emitted = true;
 	}
-      if (removed)
-	{
+      if (emitted)
 	  out << "\n";
-	  removed = 0;
-	}
     }
 
   if (ctxt->show_added_vars())
@@ -2029,6 +1954,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	out << indent << s.net_num_vars_added()
 	    << " Added variables:\n\n";
       string n;
+      bool emitted = false;
       vector<var_decl*> sorted_added_vars;
       sort_string_var_ptr_map(d.priv_->added_vars_, sorted_added_vars);
       for (vector<var_decl*>::const_iterator i =
@@ -2043,8 +1969,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 
 	  out << indent
 	      << "  ";
-	  if (total > large_num)
-	    out << "[A] ";
+	  out << "[A] ";
 	  out << "'" << n << "'";
 	  if (ctxt->show_linkage_names())
 	    {
@@ -2054,13 +1979,10 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	      out << "}";
 	    }
 	  out << "\n";
-	  ++added;
-	}
-      if (added)
-	{
-	  out << "\n";
-	  added = 0;
+	  emitted = true;
 	}
+      if (emitted)
+	out << "\n";
     }
 
   if (ctxt->show_changed_vars())
@@ -2090,16 +2012,16 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	  n1 = diff->first_subject()->get_pretty_representation();
 	  n2 = diff->second_subject()->get_pretty_representation();
 
-	  out << indent << "  [C]'" << n1 << "' was changed";
+	  out << indent << "  [C] '" << n1 << "' was changed";
 	  if (n1 != n2)
 	    out << " to '" << n2 << "'";
 	  report_loc_info(diff->second_subject(), *ctxt, out);
 	  out << ":\n";
 	  diff->report(out, indent + "    ");
+	  // Extra spacing.
 	  out << "\n";
 	}
-      if (num_changed)
-	out << "\n";
+      // Changed variables have extra spacing already. No new line here.
     }
 
   // Report removed function symbols not referenced by any debug info.
@@ -2114,6 +2036,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    << s.net_num_removed_func_syms()
 	    << " Removed function symbols not referenced by debug info:\n\n";
 
+      bool emitted = false;
       vector<elf_symbol_sptr> sorted_deleted_unrefed_fn_syms;
       sort_string_elf_symbol_map(d.priv_->deleted_unrefed_fn_syms_,
 				 sorted_deleted_unrefed_fn_syms);
@@ -2126,15 +2049,15 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    continue;
 
 	  out << indent << "  ";
-	  if (s.net_num_removed_func_syms() > large_num)
-	    out << "[D] ";
+	  out << "[D] ";
 
 	  show_linkage_name_and_aliases(out, "", **i,
 					d.first_corpus()->get_fun_symbol_map());
 	  out << "\n";
+	  emitted = true;
 	}
-      if (sorted_deleted_unrefed_fn_syms.size())
-	out << '\n';
+      if (emitted)
+	out << "\n";
     }
 
   // Report added function symbols not referenced by any debug info.
@@ -2150,6 +2073,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    << s.net_num_added_func_syms()
 	    << " Added function symbols not referenced by debug info:\n\n";
 
+      bool emitted = false;
       vector<elf_symbol_sptr> sorted_added_unrefed_fn_syms;
       sort_string_elf_symbol_map(d.priv_->added_unrefed_fn_syms_,
 				 sorted_added_unrefed_fn_syms);
@@ -2162,15 +2086,15 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    continue;
 
 	  out << indent << "  ";
-	  if (s.net_num_added_func_syms() > large_num)
-	    out << "[A] ";
+	  out << "[A] ";
 	  show_linkage_name_and_aliases(out, "",
 					**i,
 					d.second_corpus()->get_fun_symbol_map());
 	  out << "\n";
+	  emitted = true;
 	}
-      if (sorted_added_unrefed_fn_syms.size())
-	out << '\n';
+      if (emitted)
+	out << "\n";
     }
 
   // Report removed variable symbols not referenced by any debug info.
@@ -2185,6 +2109,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    << s.net_num_removed_var_syms()
 	    << " Removed variable symbols not referenced by debug info:\n\n";
 
+      bool emitted = false;
       vector<elf_symbol_sptr> sorted_deleted_unrefed_var_syms;
       sort_string_elf_symbol_map(d.priv_->deleted_unrefed_var_syms_,
 				 sorted_deleted_unrefed_var_syms);
@@ -2197,17 +2122,17 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    continue;
 
 	  out << indent << "  ";
-	  if (s.num_var_syms_removed() > large_num)
-	    out << "[D] ";
+	  out << "[D] ";
 
 	  show_linkage_name_and_aliases
 	    (out, "", **i,
 	     d.first_corpus()->get_fun_symbol_map());
 
 	  out << "\n";
+	  emitted = true;
 	}
-      if (sorted_deleted_unrefed_var_syms.size())
-	out << '\n';
+      if (emitted)
+	out << "\n";
     }
 
   // Report added variable symbols not referenced by any debug info.
@@ -2223,6 +2148,7 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    << s.net_num_added_var_syms()
 	    << " Added variable symbols not referenced by debug info:\n\n";
 
+      bool emitted = false;
       vector<elf_symbol_sptr> sorted_added_unrefed_var_syms;
       sort_string_elf_symbol_map(d.priv_->added_unrefed_var_syms_,
 				 sorted_added_unrefed_var_syms);
@@ -2235,19 +2161,18 @@ default_reporter::report(const corpus_diff& d, ostream& out,
 	    continue;
 
 	  out << indent << "  ";
-	  if (s.net_num_added_var_syms() > large_num)
-	    out << "[A] ";
+	  out << "[A] ";
 	  show_linkage_name_and_aliases(out, "", **i,
 					d.second_corpus()->get_fun_symbol_map());
 	  out << "\n";
+	  emitted = true;
 	}
-      if (sorted_added_unrefed_var_syms.size())
-	out << '\n';
+      if (emitted)
+	out << "\n";
     }
 
   // Report added/removed/changed types not reacheable from public
   // interfaces.
-
   maybe_report_unreachable_type_changes(d, s, indent, out);
 
   d.priv_->maybe_dump_diff_tree();
