@@ -60,6 +60,36 @@ combine_hashes(size_t val1, size_t val2)
   return val2;
 }
 
+/// Compute a stable string hash.
+///
+/// std::hash has no portability or stability guarantees so is
+/// unsuitable where reproducibility is a requirement such as in XML
+/// output.
+///
+/// This is the 32-bit FNV-1a algorithm. The algorithm, reference code
+/// and constants are all unencumbered. It is fast and has reasonable
+/// distribution properties.
+///
+/// https://en.wikipedia.org/wiki/Fowler-Noll-Vo_hash_function
+///
+/// @param str the string to hash.
+///
+/// @return an unsigned 32 bit hash value.
+uint32_t
+fnv_hash(const std::string& str)
+{
+  const uint32_t prime = 0x01000193;
+  const uint32_t offset_basis = 0x811c9dc5;
+  uint32_t hash = offset_basis;
+  for (std::string::const_iterator i = str.begin(); i != str.end(); ++i)
+    {
+      uint8_t byte = *i;
+      hash = hash ^ byte;
+      hash = hash * prime;
+    }
+  return hash;
+}
+
 }//end namespace hashing
 
 using std::list;
@@ -605,7 +635,8 @@ class_or_union::hash::operator()(const class_or_union& t) const
   if (t.get_is_declaration_only())
     {
       ABG_ASSERT(t.get_definition_of_declaration());
-      size_t v = operator()(*t.get_definition_of_declaration());
+      size_t v = operator()
+	(*is_class_or_union_type(t.get_definition_of_declaration()));
       return v;
     }
 
@@ -960,6 +991,11 @@ operator()(const shared_ptr<class_tdecl> t) const
 /// inheritance hierarchy, make sure to handle the most derived type
 /// first.
 ///
+/// FIXME: This hashing function is not maintained and is surely
+/// broken in subtle ways.  In pratice, the various *::hash functors
+/// should be audited before they are used here.  They should all
+/// match what is done in the 'equals' functions in abg-ir.cc.
+///
 /// @param t a pointer to the type declaration to be hashed
 ///
 /// @return the resulting hash
