@@ -45,6 +45,42 @@ bool
 leaf_reporter::diff_to_be_reported(const diff *d) const
 {return d && d->to_be_reported() && d->has_local_changes();}
 
+/// Test if a given instance of @ref corpus_diff carries changes whose
+/// reports are not suppressed by any suppression specification.  In
+/// effect, these are deemed incompatible ABI changes.
+///
+/// @param d the @ref corpus_diff to consider
+///
+/// @return true iff @p d carries subtype changes that are deemed
+/// incompatible ABI changes.
+bool
+leaf_reporter::diff_has_net_changes(const corpus_diff *d) const
+{
+  if (!d)
+    return false;
+
+  const corpus_diff::diff_stats& stats = const_cast<corpus_diff*>(d)->
+    apply_filters_and_suppressions_before_reporting();
+
+  // Logic here should match emit_diff_stats.
+  return (d->architecture_changed()
+	  || d->soname_changed()
+	  || stats.net_num_func_removed()
+	  || stats.net_num_leaf_type_changes()
+	  || stats.net_num_leaf_func_changes()
+	  || stats.net_num_func_added()
+	  || stats.net_num_vars_removed()
+	  || stats.net_num_leaf_var_changes()
+	  || stats.net_num_vars_added()
+	  || stats.net_num_removed_unreachable_types()
+	  || stats.net_num_changed_unreachable_types()
+	  || stats.net_num_added_unreachable_types()
+	  || stats.net_num_removed_func_syms()
+	  || stats.net_num_added_func_syms()
+	  || stats.net_num_removed_var_syms()
+	  || stats.net_num_added_var_syms());
+}
+
 /// Report the changes carried by the diffs contained in an instance
 /// of @ref string_diff_ptr_map.
 ///
@@ -74,11 +110,11 @@ report_diffs(const reporter_base& r,
       if (r.diff_to_be_reported((*i)->get_canonical_diff()))
 	{
 	  if (started_to_emit)
-	    out << "\n\n";
+	    out << "\n";
 
 	  string n = (*i)->first_subject()->get_pretty_representation();
 
-	  out << indent << "'" << n ;
+	  out << indent << "'" << n;
 
 	  report_loc_info((*i)->first_subject(),
 			  *(*i)->context(), out);
@@ -86,7 +122,6 @@ report_diffs(const reporter_base& r,
 	  out << "' changed:\n";
 
 	  (*i)->get_canonical_diff()->report(out, indent + "  ");
-	  out << "\n";
 	  started_to_emit = true;
 	}
     }
@@ -167,7 +202,9 @@ leaf_reporter::report(const typedef_diff& d,
   if (!diff_to_be_reported(&d))
     return;
 
-  report_local_typedef_changes(d, out, indent);
+  // all changes carried by a typedef_diff are considered local, so
+  // let's just call the default reporter here.
+  default_reporter::report(d, out, indent);
 
   maybe_report_interfaces_impacted_by_diff(&d, out, indent);
 }
@@ -250,7 +287,7 @@ leaf_reporter::report(const fn_parm_diff& d,
   out << " of type '"
       << f->get_type_pretty_representation()
       << "' changed:\n";
-  d.type_diff()->report(out, indent);
+  d.type_diff()->report(out, indent + "  ");
 }
 
 /// Report the changes carried by a @ref function_type_diff node.
@@ -333,8 +370,8 @@ leaf_reporter::report(const scope_diff& d,
 
       out << indent << "  '"
 	  << (*dif)->first_subject()->get_pretty_representation()
-          << "' was changed to '"
-          << (*dif)->second_subject()->get_pretty_representation() << "'";
+	  << "' was changed to '"
+	  << (*dif)->second_subject()->get_pretty_representation() << "'";
       report_loc_info((*dif)->second_subject(), *d.context(), out);
       out << ":\n";
 
@@ -428,8 +465,7 @@ leaf_reporter::report(const array_diff& d,
   report_name_size_and_alignment_changes(d.first_array(),
 					 d.second_array(),
 					 d.context(),
-					 out, indent,
-					 /*new line=*/false);
+					 out, indent);
 
   diff_sptr dif = d.element_type_diff();
   if (diff_to_be_reported(dif.get()))
@@ -441,8 +477,6 @@ leaf_reporter::report(const array_diff& d,
       dif->report(out, indent + "  ");
     }
 
-  report_loc_info(d.second_array(), *d.context(), out);
-
   maybe_report_interfaces_impacted_by_diff(&d, out, indent);
 }
 
@@ -465,7 +499,7 @@ leaf_reporter::report(const class_or_union_diff& d,
   const diff_context_sptr& ctxt = d.context();
 
   // Report class decl-only -> definition change.
-  if (ctxt->get_allowed_category() & CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY)
+  if (ctxt->get_allowed_category() & TYPE_DECL_ONLY_DEF_CHANGE_CATEGORY)
     if (filtering::has_class_decl_only_def_change(first, second))
       {
 	string was =
@@ -479,7 +513,7 @@ leaf_reporter::report(const class_or_union_diff& d,
 	  : " and is now a defined type";
 
 	out << indent << "type " << first->get_pretty_representation()
-	    << was << is_now;
+	    << was << is_now << "\n";
 	return;
       }
 
@@ -493,7 +527,6 @@ leaf_reporter::report(const class_or_union_diff& d,
       if (numdels)
 	report_mem_header(out, numdels, num_filtered, del_kind,
 			  "member function", indent);
-      bool emitted = false;
       for (string_member_function_sptr_map::const_iterator i =
 	     d.get_priv()->deleted_member_functions_.begin();
 	   i != d.get_priv()->deleted_member_functions_.end();
@@ -504,16 +537,10 @@ leaf_reporter::report(const class_or_union_diff& d,
 	      && !get_member_function_is_virtual(i->second))
 	    continue;
 
-	  if (emitted
-	      && i != d.get_priv()->deleted_member_functions_.begin())
-	    out << "\n";
 	  method_decl_sptr mem_fun = i->second;
 	  out << indent << "  ";
 	  represent(*ctxt, mem_fun, out);
-	  emitted = true;
 	}
-      if (emitted)
-	out << "\n";
 
       // report insertions;
       int numins = d.get_priv()->inserted_member_functions_.size();
@@ -521,7 +548,6 @@ leaf_reporter::report(const class_or_union_diff& d,
       if (numins)
 	report_mem_header(out, numins, num_filtered, ins_kind,
 			  "member function", indent);
-      emitted = false;
       for (string_member_function_sptr_map::const_iterator i =
 	     d.get_priv()->inserted_member_functions_.begin();
 	   i != d.get_priv()->inserted_member_functions_.end();
@@ -532,22 +558,15 @@ leaf_reporter::report(const class_or_union_diff& d,
 	      && !get_member_function_is_virtual(i->second))
 	    continue;
 
-	  if (emitted
-	      && i != d.get_priv()->inserted_member_functions_.begin())
-	    out << "\n";
 	  method_decl_sptr mem_fun = i->second;
 	  out << indent << "  ";
 	  represent(*ctxt, mem_fun, out);
-	  emitted = true;
 	}
-      if (emitted)
-	out << "\n";
 
       // report member function with sub-types changes
       int numchanges = d.get_priv()->sorted_changed_member_functions_.size();
       if (numchanges)
 	report_mem_header(out, change_kind, "member function", indent);
-      emitted = false;
       for (function_decl_diff_sptrs_type::const_iterator i =
 	     d.get_priv()->sorted_changed_member_functions_.begin();
 	   i != d.get_priv()->sorted_changed_member_functions_.end();
@@ -567,15 +586,9 @@ leaf_reporter::report(const class_or_union_diff& d,
 
 	  string repr =
 	    (*i)->first_function_decl()->get_pretty_representation();
-	  if (emitted
-	      && i != d.get_priv()->sorted_changed_member_functions_.begin())
-	    out << "\n";
 	  out << indent << "  '" << repr << "' has some changes:\n";
 	  diff->report(out, indent + "    ");
-	  emitted = true;
 	}
-      if (numchanges)
-	out << "\n";
     }
 
   // data members
@@ -592,7 +605,6 @@ leaf_reporter::report(const class_or_union_diff& d,
 	  sort_data_members
 	    (d.class_or_union_diff::get_priv()->deleted_data_members_,
 	     sorted_dms);
-	  bool emitted = false;
 	  for (vector<decl_base_sptr>::const_iterator i = sorted_dms.begin();
 	       i != sorted_dms.end();
 	       ++i)
@@ -602,14 +614,8 @@ leaf_reporter::report(const class_or_union_diff& d,
 	      ABG_ASSERT(data_mem);
 	      if (get_member_is_static(data_mem))
 		continue;
-	      if (emitted)
-		out << "\n";
-	      out << indent << "  ";
-	      represent_data_member(data_mem, ctxt, out);
-	      emitted = true;
+	      represent_data_member(data_mem, ctxt, out, indent + "  ");
 	    }
-	  if (emitted)
-	    out << "\n";
 	}
 
       //report insertions
@@ -630,67 +636,43 @@ leaf_reporter::report(const class_or_union_diff& d,
 	      var_decl_sptr data_mem =
 		dynamic_pointer_cast<var_decl>(*i);
 	      ABG_ASSERT(data_mem);
-	      out << indent << "  ";
-	      represent_data_member(data_mem, ctxt, out);
+	      represent_data_member(data_mem, ctxt, out, indent + "  ");
 	    }
 	}
 
       // report changes
-      size_t numchanges =
-	d.class_or_union_diff::get_priv()->sorted_changed_dm_.size();
+      size_t numchanges = (d.sorted_changed_data_members().size()
+			   + d.sorted_subtype_changed_data_members().size());
+
       size_t num_filtered =
-	d.class_or_union_diff::get_priv()->
-	count_filtered_changed_dm(/*local_only =*/ true);
+	(d.count_filtered_changed_data_members(/*local_only=*/true)
+	 + d.count_filtered_subtype_changed_data_members(/*local_only=*/true));
+
       ABG_ASSERT(numchanges >= num_filtered);
       size_t net_numchanges = numchanges - num_filtered;
 
-      bool emitted_data_members_changes = false;
       if (net_numchanges)
 	{
-	  report_mem_header(out, subtype_change_kind, "data member", indent);
+	  report_mem_header(out, change_kind, "data member", indent);
+
 	  for (var_diff_sptrs_type::const_iterator it =
-		 d.class_or_union_diff::get_priv()->
-		 sorted_changed_dm_.begin();
-	       it != d.class_or_union_diff::get_priv()->
-		 sorted_changed_dm_.end();
+		 d.sorted_changed_data_members().begin();
+	       it != d.sorted_changed_data_members().end();
 	       ++it)
-	    {
-	      if (diff_to_be_reported((*it).get()))
-		{
-		  represent(*it, ctxt, out, indent + " ",
-			    /*local_only=*/true);
-		  out << "\n";
-		  emitted_data_members_changes = true;
-		}
-	    }
+	    if (diff_to_be_reported((*it).get()))
+	      represent(*it, ctxt, out, indent + "  ", /*local_only=*/true);
+
+	  for (var_diff_sptrs_type::const_iterator it =
+		 d.sorted_subtype_changed_data_members().begin();
+	       it != d.sorted_subtype_changed_data_members().end();
+	       ++it)
+	    if (diff_to_be_reported((*it).get()))
+	      represent(*it, ctxt, out, indent + "  ", /*local_only=*/true);
 	}
 
-      numchanges =
-	d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.size();
-      num_filtered =
-	d.class_or_union_diff::get_priv()->
-	count_filtered_subtype_changed_dm(/*local_only =*/ true);
-      ABG_ASSERT(numchanges >= num_filtered);
-      net_numchanges = numchanges - num_filtered;
-
-      if (net_numchanges)
-	{
-	  if (!emitted_data_members_changes)
-	    report_mem_header(out, change_kind, "data member", indent);
-	  for (var_diff_sptrs_type::const_iterator it =
-		 d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.begin();
-	       it != d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.end();
-	       ++it)
-	    {
-	      if (diff_to_be_reported((*it).get()))
-		{
-		  represent(*it, ctxt, out, indent + " ",
-			    /*local_only=*/true);
-		  out << "\n";
-		  emitted_data_members_changes = true;
-		}
-	    }
-	}
+      // Report about data members replaced by an anonymous union data
+      // member.
+      maybe_report_data_members_replaced_by_anon_dm(d, out, indent);
     }
 }
 
@@ -716,10 +698,8 @@ leaf_reporter::report(const class_diff& d,
   class_decl_sptr first = d.first_class_decl(),
     second = d.second_class_decl();
 
-  if (report_name_size_and_alignment_changes(first, second, d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second, d.context(),
+					 out, indent);
 
   const diff_context_sptr& ctxt = d.context();
   maybe_report_diff_for_member(first, second, ctxt, out, indent);
@@ -750,10 +730,8 @@ leaf_reporter::report(const union_diff& d,
   // Now report the changes about the differents parts of the type.
   union_decl_sptr first = d.first_union_decl(), second = d.second_union_decl();
 
-  if (report_name_size_and_alignment_changes(first, second, d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second, d.context(),
+					 out, indent);
 
   maybe_report_diff_for_member(first, second,d. context(), out, indent);
 
@@ -789,10 +767,7 @@ leaf_reporter::report(const distinct_diff& d,
   report_loc_info(s, *d.context(), out);
   out << "\n";
 
-  if (report_size_and_alignment_changes(f, s, d.context(), out, indent,
-					/*start_with_new_line=*/false))
-    out << "\n";
-
+  report_size_and_alignment_changes(f, s, d.context(), out, indent);
   maybe_report_interfaces_impacted_by_diff(&d, out, indent);
 }
 
@@ -982,11 +957,9 @@ leaf_reporter::report(const var_diff& d,
   decl_base_sptr first = d.first_var(), second = d.second_var();
   string n = first->get_pretty_representation();
 
-  if (report_name_size_and_alignment_changes(first, second,
-					     d.context(),
-					     out, indent,
-					     /*start_with_new_line=*/false))
-    out << "\n";
+  report_name_size_and_alignment_changes(first, second,
+					 d.context(),
+					 out, indent);
 
   maybe_report_diff_for_symbol(d.first_var()->get_symbol(),
 			       d.second_var()->get_symbol(),
@@ -1000,7 +973,7 @@ leaf_reporter::report(const var_diff& d,
 	{
 	  RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER2(dif, "type");
 	  out << indent << "type of variable changed:\n";
-	  dif->report(out, indent + " ");
+	  dif->report(out, indent + "  ");
 	}
     }
 }
@@ -1034,17 +1007,15 @@ leaf_reporter::report(const corpus_diff& d,
   if (!d.has_changes())
     return;
 
-  const diff_context_sptr& ctxt = d.context();
-
-  size_t removed = 0, added = 0;
   const corpus_diff::diff_stats &s =
     const_cast<corpus_diff&>(d).
     apply_filters_and_suppressions_before_reporting();
 
+  const diff_context_sptr& ctxt = d.context();
+
   d.priv_->emit_diff_stats(s, out, indent);
   if (ctxt->show_stats_only())
     return;
-
   out << "\n";
 
   if (ctxt->show_soname_change()
@@ -1059,6 +1030,7 @@ leaf_reporter::report(const corpus_diff& d,
 	<< d.first_corpus()->get_architecture_name() << "' to '"
 	<< d.second_corpus()->get_architecture_name() << "'\n\n";
 
+  /// Report removed/added/changed functions.
   if (ctxt->show_deleted_fns())
     {
       if (s.net_num_func_removed() == 1)
@@ -1066,6 +1038,7 @@ leaf_reporter::report(const corpus_diff& d,
       else if (s.net_num_func_removed() > 1)
 	out << indent << s.net_num_func_removed() << " Removed functions:\n\n";
 
+      bool emitted = false;
       vector<function_decl*>sorted_deleted_fns;
       sort_string_function_ptr_map(d.priv_->deleted_fns_, sorted_deleted_fns);
       for (vector<function_decl*>::const_iterator i =
@@ -1098,85 +1071,12 @@ leaf_reporter::report(const corpus_diff& d,
 		  << c->get_pretty_representation()
 		  << "\n";
 	    }
-	  ++removed;
+	  emitted = true;
 	}
-      if (removed)
+      if (emitted)
 	out << "\n";
     }
 
-  if (ctxt->show_changed_fns())
-    {
-      // Show changed functions.
-      size_t num_changed = s.net_num_leaf_func_changes();
-      if (num_changed == 1)
-	out << indent << "1 function with some sub-type change:\n\n";
-      else if (num_changed > 1)
-	out << indent << num_changed
-	    << " functions with some sub-type change:\n\n";
-
-      bool changed = false;
-      vector<function_decl_diff_sptr> sorted_changed_fns;
-      sort_string_function_decl_diff_sptr_map(d.priv_->changed_fns_map_,
-					      sorted_changed_fns);
-      for (vector<function_decl_diff_sptr>::const_iterator i =
-	     sorted_changed_fns.begin();
-	   i != sorted_changed_fns.end();
-	   ++i)
-	{
-	  diff_sptr diff = *i;
-	  if (!diff)
-	    continue;
-
-	  if (diff_to_be_reported(diff.get()))
-	    {
-	      function_decl_sptr fn = (*i)->first_function_decl();
-	      out << indent << "  [C]'"
-		  << fn->get_pretty_representation() << "'";
-	      report_loc_info((*i)->second_function_decl(), *ctxt, out);
-	      out << " has some sub-type changes:\n";
-	      if ((fn->get_symbol()->has_aliases()
-		   && !(is_member_function(fn)
-			&& get_member_function_is_ctor(fn))
-		   && !(is_member_function(fn)
-			&& get_member_function_is_dtor(fn)))
-		  || (is_c_language(get_translation_unit(fn)->get_language())
-		      && fn->get_name() != fn->get_linkage_name()))
-		{
-		  int number_of_aliases =
-		    fn->get_symbol()->get_number_of_aliases();
-		  if (number_of_aliases == 0)
-		    {
-		      out << indent << "    "
-			  << "Please note that the exported symbol of "
-			"this function is "
-			  << fn->get_symbol()->get_id_string()
-			  << "\n";
-		    }
-		  else
-		    {
-		      out << indent << "    "
-			  << "Please note that the symbol of this function is "
-			  << fn->get_symbol()->get_id_string()
-			  << "\n     and it aliases symbol";
-		      if (number_of_aliases > 1)
-			out << "s";
-		      out << ": "
-			  << fn->get_symbol()->get_aliases_id_string(false)
-			  << "\n";
-		    }
-		}
-	      diff->report(out, indent + "    ");
-	      out << "\n";
-	      changed |= true;
-	    }
-	}
-      if (changed)
-	{
-	  out << "\n";
-	  changed = false;
-	}
-    }
-
   if (ctxt->show_added_fns())
     {
       if (s.net_num_func_added() == 1)
@@ -1184,6 +1084,7 @@ leaf_reporter::report(const corpus_diff& d,
       else if (s.net_num_func_added() > 1)
 	out << indent << s.net_num_func_added()
 	    << " Added functions:\n\n";
+      bool emitted = false;
       vector<function_decl*> sorted_added_fns;
       sort_string_function_ptr_map(d.priv_->added_fns_, sorted_added_fns);
       for (vector<function_decl*>::const_iterator i = sorted_added_fns.begin();
@@ -1219,13 +1120,343 @@ leaf_reporter::report(const corpus_diff& d,
 		  << c->get_pretty_representation()
 		  << "\n";
 	    }
-	  ++added;
+	  emitted = true;
 	}
-      if (added)
+      if (emitted)
+	out << "\n";
+    }
+
+  if (ctxt->show_changed_fns())
+    {
+      // Show changed functions.
+      size_t num_changed = s.net_num_leaf_func_changes();
+      if (num_changed == 1)
+	out << indent << "1 function with some sub-type change:\n\n";
+      else if (num_changed > 1)
+	out << indent << num_changed
+	    << " functions with some sub-type change:\n\n";
+
+      vector<function_decl_diff_sptr> sorted_changed_fns;
+      sort_string_function_decl_diff_sptr_map(d.priv_->changed_fns_map_,
+					      sorted_changed_fns);
+      for (vector<function_decl_diff_sptr>::const_iterator i =
+	     sorted_changed_fns.begin();
+	   i != sorted_changed_fns.end();
+	   ++i)
 	{
-	  out << "\n";
-	  added = false;
+	  diff_sptr diff = *i;
+	  if (!diff)
+	    continue;
+
+	  if (diff_to_be_reported(diff.get()))
+	    {
+	      function_decl_sptr fn = (*i)->first_function_decl();
+	      out << indent << "  [C] '"
+		  << fn->get_pretty_representation() << "'";
+	      report_loc_info((*i)->second_function_decl(), *ctxt, out);
+	      out << " has some sub-type changes:\n";
+	      if ((fn->get_symbol()->has_aliases()
+		   && !(is_member_function(fn)
+			&& get_member_function_is_ctor(fn))
+		   && !(is_member_function(fn)
+			&& get_member_function_is_dtor(fn)))
+		  || (is_c_language(get_translation_unit(fn)->get_language())
+		      && fn->get_name() != fn->get_linkage_name()))
+		{
+		  int number_of_aliases =
+		    fn->get_symbol()->get_number_of_aliases();
+		  if (number_of_aliases == 0)
+		    {
+		      out << indent << "    "
+			  << "Please note that the exported symbol of "
+			"this function is "
+			  << fn->get_symbol()->get_id_string()
+			  << "\n";
+		    }
+		  else
+		    {
+		      out << indent << "    "
+			  << "Please note that the symbol of this function is "
+			  << fn->get_symbol()->get_id_string()
+			  << "\n     and it aliases symbol";
+		      if (number_of_aliases > 1)
+			out << "s";
+		      out << ": "
+			  << fn->get_symbol()->get_aliases_id_string(false)
+			  << "\n";
+		    }
+		}
+	      diff->report(out, indent + "    ");
+	      // Extra spacing.
+	      out << "\n";
+	    }
 	}
+      // Changed functions have extra spacing already. No new line here.
+    }
+
+  // Report removed/added/changed variables.
+  if (ctxt->show_deleted_vars())
+    {
+      if (s.net_num_vars_removed() == 1)
+	out << indent << "1 Removed variable:\n\n";
+      else if (s.net_num_vars_removed() > 1)
+	out << indent << s.net_num_vars_removed()
+	    << " Removed variables:\n\n";
+      string n;
+      bool emitted = false;
+      vector<var_decl*> sorted_deleted_vars;
+      sort_string_var_ptr_map(d.priv_->deleted_vars_, sorted_deleted_vars);
+      for (vector<var_decl*>::const_iterator i =
+	     sorted_deleted_vars.begin();
+	   i != sorted_deleted_vars.end();
+	   ++i)
+	{
+	  if (d.priv_->deleted_variable_is_suppressed(*i))
+	    continue;
+
+	  n = (*i)->get_pretty_representation();
+
+	  out << indent
+	      << "  ";
+	  out << "[D] ";
+	  out << "'"
+	      << n
+	      << "'";
+	  if (ctxt->show_linkage_names())
+	    {
+	      out << "    {";
+	      show_linkage_name_and_aliases(out, "", *(*i)->get_symbol(),
+					    d.first_corpus()->get_var_symbol_map());
+	      out << "}";
+	    }
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
+    }
+
+  if (ctxt->show_added_vars())
+    {
+      if (s.net_num_vars_added() == 1)
+	out << indent << "1 Added variable:\n\n";
+      else if (s.net_num_vars_added() > 1)
+	out << indent << s.net_num_vars_added()
+	    << " Added variables:\n\n";
+      string n;
+      bool emitted = false;
+      vector<var_decl*> sorted_added_vars;
+      sort_string_var_ptr_map(d.priv_->added_vars_, sorted_added_vars);
+      for (vector<var_decl*>::const_iterator i =
+	     sorted_added_vars.begin();
+	   i != sorted_added_vars.end();
+	   ++i)
+	{
+	  if (d.priv_->added_variable_is_suppressed(*i))
+	    continue;
+
+	  n = (*i)->get_pretty_representation();
+
+	  out << indent
+	      << "  ";
+	  out << "[A] ";
+	  out << "'" << n << "'";
+	  if (ctxt->show_linkage_names())
+	    {
+	      out << "    {";
+	      show_linkage_name_and_aliases(out, "", *(*i)->get_symbol(),
+					    d.second_corpus()->get_var_symbol_map());
+	      out << "}";
+	    }
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
+    }
+
+  if (ctxt->show_changed_vars())
+    {
+      size_t num_changed = s.net_num_leaf_var_changes();
+      if (num_changed == 1)
+	out << indent << "1 Changed variable:\n\n";
+      else if (num_changed > 1)
+	out << indent << num_changed
+	    << " Changed variables:\n\n";
+      string n1, n2;
+      for (var_diff_sptrs_type::const_iterator i =
+	     d.priv_->sorted_changed_vars_.begin();
+	   i != d.priv_->sorted_changed_vars_.end();
+	   ++i)
+	{
+	  diff_sptr diff = *i;
+
+	  if (!diff)
+	    continue;
+
+	  if (!diff_to_be_reported(diff.get()))
+	    continue;
+
+	  n1 = diff->first_subject()->get_pretty_representation();
+	  n2 = diff->second_subject()->get_pretty_representation();
+
+	  out << indent << "  [C] '" << n1 << "' was changed";
+	  if (n1 != n2)
+	    out << " to '" << n2 << "'";
+	  report_loc_info(diff->second_subject(), *ctxt, out);
+	  out << ":\n";
+	  diff->report(out, indent + "    ");
+	  // Extra spacing.
+	  out << "\n";
+	}
+      // Changed variables have extra spacing already. No new line here.
+    }
+
+  // Report removed function symbols not referenced by any debug info.
+  if (ctxt->show_symbols_unreferenced_by_debug_info()
+      && d.priv_->deleted_unrefed_fn_syms_.size())
+    {
+      if (s.net_num_removed_func_syms() == 1)
+	out << indent
+	    << "1 Removed function symbol not referenced by debug info:\n\n";
+      else if (s.net_num_removed_func_syms() > 0)
+	out << indent
+	    << s.net_num_removed_func_syms()
+	    << " Removed function symbols not referenced by debug info:\n\n";
+
+      bool emitted = false;
+      vector<elf_symbol_sptr> sorted_deleted_unrefed_fn_syms;
+      sort_string_elf_symbol_map(d.priv_->deleted_unrefed_fn_syms_,
+				 sorted_deleted_unrefed_fn_syms);
+      for (vector<elf_symbol_sptr>::const_iterator i =
+	     sorted_deleted_unrefed_fn_syms.begin();
+	   i != sorted_deleted_unrefed_fn_syms.end();
+	   ++i)
+	{
+	  if (d.priv_->deleted_unrefed_fn_sym_is_suppressed((*i).get()))
+	    continue;
+
+	  out << indent << "  ";
+	  out << "[D] ";
+
+	  show_linkage_name_and_aliases(out, "", **i,
+					d.first_corpus()->get_fun_symbol_map());
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
+    }
+
+  // Report added function symbols not referenced by any debug info.
+  if (ctxt->show_symbols_unreferenced_by_debug_info()
+      && ctxt->show_added_symbols_unreferenced_by_debug_info()
+      && d.priv_->added_unrefed_fn_syms_.size())
+    {
+      if (s.net_num_added_func_syms() == 1)
+	out << indent
+	    << "1 Added function symbol not referenced by debug info:\n\n";
+      else if (s.net_num_added_func_syms() > 0)
+	out << indent
+	    << s.net_num_added_func_syms()
+	    << " Added function symbols not referenced by debug info:\n\n";
+
+      bool emitted = false;
+      vector<elf_symbol_sptr> sorted_added_unrefed_fn_syms;
+      sort_string_elf_symbol_map(d.priv_->added_unrefed_fn_syms_,
+				 sorted_added_unrefed_fn_syms);
+      for (vector<elf_symbol_sptr>::const_iterator i =
+	     sorted_added_unrefed_fn_syms.begin();
+	   i != sorted_added_unrefed_fn_syms.end();
+	   ++i)
+	{
+	  if (d.priv_->added_unrefed_fn_sym_is_suppressed((*i).get()))
+	    continue;
+
+	  out << indent << "  ";
+	  out << "[A] ";
+	  show_linkage_name_and_aliases(out, "",
+					**i,
+					d.second_corpus()->get_fun_symbol_map());
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
+    }
+
+  // Report removed variable symbols not referenced by any debug info.
+  if (ctxt->show_symbols_unreferenced_by_debug_info()
+      && d.priv_->deleted_unrefed_var_syms_.size())
+    {
+      if (s.net_num_removed_var_syms() == 1)
+	out << indent
+	    << "1 Removed variable symbol not referenced by debug info:\n\n";
+      else if (s.net_num_removed_var_syms() > 0)
+	out << indent
+	    << s.net_num_removed_var_syms()
+	    << " Removed variable symbols not referenced by debug info:\n\n";
+
+      bool emitted = false;
+      vector<elf_symbol_sptr> sorted_deleted_unrefed_var_syms;
+      sort_string_elf_symbol_map(d.priv_->deleted_unrefed_var_syms_,
+				 sorted_deleted_unrefed_var_syms);
+      for (vector<elf_symbol_sptr>::const_iterator i =
+	     sorted_deleted_unrefed_var_syms.begin();
+	   i != sorted_deleted_unrefed_var_syms.end();
+	   ++i)
+	{
+	  if (d.priv_->deleted_unrefed_var_sym_is_suppressed((*i).get()))
+	    continue;
+
+	  out << indent << "  ";
+	  out << "[D] ";
+
+	  show_linkage_name_and_aliases
+	    (out, "", **i,
+	     d.first_corpus()->get_fun_symbol_map());
+
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
+    }
+
+  // Report added variable symbols not referenced by any debug info.
+  if (ctxt->show_symbols_unreferenced_by_debug_info()
+      && ctxt->show_added_symbols_unreferenced_by_debug_info()
+      && d.priv_->added_unrefed_var_syms_.size())
+    {
+      if (s.net_num_added_var_syms() == 1)
+	out << indent
+	    << "1 Added variable symbol not referenced by debug info:\n\n";
+      else if (s.net_num_added_var_syms() > 0)
+	out << indent
+	    << s.net_num_added_var_syms()
+	    << " Added variable symbols not referenced by debug info:\n\n";
+
+      bool emitted = false;
+      vector<elf_symbol_sptr> sorted_added_unrefed_var_syms;
+      sort_string_elf_symbol_map(d.priv_->added_unrefed_var_syms_,
+				 sorted_added_unrefed_var_syms);
+      for (vector<elf_symbol_sptr>::const_iterator i =
+	     sorted_added_unrefed_var_syms.begin();
+	   i != sorted_added_unrefed_var_syms.end();
+	   ++i)
+	{
+	  if (d.priv_->added_unrefed_var_sym_is_suppressed((*i).get()))
+	    continue;
+
+	  out << indent << "  ";
+	  out << "[A] ";
+	  show_linkage_name_and_aliases(out, "", **i,
+					d.second_corpus()->get_fun_symbol_map());
+	  out << "\n";
+	  emitted = true;
+	}
+      if (emitted)
+	out << "\n";
     }
 
   // Now show the changed types.
