@@ -242,16 +242,19 @@ represent(const diff_context& ctxt,
 /// @param ctxt the current diff context.
 ///
 /// @param out the output stream to send the representation to
+///
+/// @param indent the indentation string to use for the change report.
 void
 represent_data_member(var_decl_sptr d,
 		      const diff_context_sptr& ctxt,
-		      ostream& out)
+		      ostream& out,
+		      const string& indent)
 {
   if (!is_data_member(d)
       || (!get_member_is_static(d) && !get_data_member_is_laid_out(d)))
     return;
 
-  out << "'" << d->get_pretty_representation() << "'";
+  out << indent << "'" << d->get_pretty_representation() << "'";
   if (!get_member_is_static(d))
     {
       // Do not emit offset information for data member of a union
@@ -262,8 +265,8 @@ represent_data_member(var_decl_sptr d,
 			    get_data_member_offset(d),
 			    *ctxt, out);
       report_loc_info(d, *ctxt, out);
-      out << "\n";
     }
+  out << "\n";
 }
 
 /// If a given @ref var_diff node carries a data member change in
@@ -406,125 +409,143 @@ represent(const var_diff_sptr	&diff,
   if (!ctxt->get_reporter()->diff_to_be_reported(diff.get()))
     return;
 
-  var_decl_sptr o = diff->first_var();
-  var_decl_sptr n = diff->second_var();
+  const var_decl_sptr o = diff->first_var();
+  const var_decl_sptr n = diff->second_var();
+  const bool o_anon = !!is_anonymous_data_member(o);
+  const bool n_anon = !!is_anonymous_data_member(n);
+  const bool is_strict_anonymous_data_member_change = o_anon && n_anon;
+  const string o_name = o->get_qualified_name();
+  const string n_name = n->get_qualified_name();
+  const uint64_t o_size = get_var_size_in_bits(o);
+  const uint64_t n_size = get_var_size_in_bits(n);
+  const uint64_t o_offset = get_data_member_offset(o);
+  const uint64_t n_offset = get_data_member_offset(n);
+  const string o_pretty_representation = o->get_pretty_representation();
+  // no n_pretty_representation here as it's only needed in a couple of places
+  const bool show_size_offset_changes = ctxt->get_allowed_category()
+					& SIZE_OR_OFFSET_CHANGE_CATEGORY;
 
+  // Has the main diff text been output?
   bool emitted = false;
+  // Are we continuing on a new line? (implies emitted)
   bool begin_with_and = false;
-  string name1 = o->get_qualified_name();
-  string name2 = n->get_qualified_name();
-  string pretty_representation = o->get_pretty_representation();
-  bool is_strict_anonymous_data_member_change = false;
+  // Have we reported a size change already?
+  bool size_reported = false;
 
-  if (is_anonymous_data_member(o) && is_anonymous_data_member(n))
+  //----------------------------------------------------------------
+  // First we'll try to emit a report about the type change of this
+  // var_decl_diff.
+  //
+  // In the context of that type change report, we need to keep in
+  // mind that because we want to emit specific (useful) reports about
+  // anonymous data member changes, we'll try to detect the various
+  // scenarii that involve anonymous data member changes.
+  //
+  // Then, as a fallback method, we'll emit a more generic type change
+  // report for the other generic type changes.
+  //----------------------------------------------------------------
+
+  if (is_strict_anonymous_data_member_change)
     {
-      is_strict_anonymous_data_member_change = true;
-      string tr1 = o->get_pretty_representation();
-      string tr2 = n->get_pretty_representation();
-      type_base_sptr t1 = o->get_type(), t2 = n->get_type();
-      if (tr1 != tr2)
+      const string n_pretty_representation = n->get_pretty_representation();
+      const type_base_sptr o_type = o->get_type(), n_type = n->get_type();
+      if (o_pretty_representation != n_pretty_representation)
 	{
 	  show_offset_or_size(indent + "anonymous data member at offset",
-			      get_data_member_offset(o),
-			      *ctxt, out);
+			      o_offset, *ctxt, out);
 
 	  out << " changed from:\n"
-	      << indent << "  " << tr1 << "\n"
+	      << indent << "  " << o_pretty_representation << "\n"
 	      << indent << "to:\n"
-	      << indent << "  " << tr2 << "\n";
-	}
-      else if (get_type_name(t1) != get_type_name(t2)
-	       && is_decl(t1) && is_decl(t2)
-	       && is_decl(t1)->get_is_anonymous()
-	       && is_decl(t2)->get_is_anonymous())
-	{
-	  out << indent << "while looking at anonymous data member '"
-	      << tr1 << "':\n"
-	      << indent << "the internal name of that anonymous data member"
-	                   "changed from:\n"
-	      << indent << " " << get_type_name(o->get_type()) << "\n"
-	      << indent << "to:\n"
-	      << indent << " " << get_type_name(n->get_type()) << "\n"
-	      << indent << " This is usually due to "
-	      <<"an anonymous member type being added or removed from "
-	      << "the containing type\n";
-	}
-    }
-  else if (filtering::has_anonymous_data_member_change(diff))
-    {
-      // So we are looking at a non-anonymous data member change from
-      // or to anonymous data member.
-      if (is_anonymous_data_member(o))
-	{
-	  string repr = "anonymous data member "
-	    + o->get_pretty_representation()
-	    + " at offset";
-
-	  show_offset_or_size(indent + repr,
-			      get_data_member_offset(o),
-			      *ctxt, out);
-	  out << " became data member '"
-	      << n->get_pretty_representation()
-	      << "'";
-	}
-      else
-	{
-	  ABG_ASSERT(is_anonymous_data_member(n));
-	  string repr = "data member "
-	    + o->get_pretty_representation()
-	    + " at offset";
-
-	  show_offset_or_size(indent + repr,
-			      get_data_member_offset(o),
-			      *ctxt, out);
-	  out << " became anonymous data member '"
-	      << n->get_pretty_representation()
-	      << "'";
-	}
-    }
-  else if (diff_sptr d = diff->type_diff())
-    {
-      if (local_only && d->has_local_changes())
-	{
-	  out << indent << "type '" << get_pretty_representation(o->get_type())
-	      << "' of '" << o->get_qualified_name()
-	      << "' changed";
-
-	  if (d->currently_reporting())
-	    {
-	      out << " as being reported\n";
-	    }
-	  else if (d->reported_once())
-	    {
-	      out << " as reported earlier\n";
-	    }
-	  else
-	    {
-	      out << ":\n";
-	      d->report(out, indent + "  ");
-	    }
+	      << indent << "  " << n_pretty_representation << "\n";
 
 	  begin_with_and = true;
 	  emitted = true;
 	}
-      else
+      else if (get_type_name(o_type) != get_type_name(n_type)
+	       && is_decl(o_type) && is_decl(n_type)
+	       && is_decl(o_type)->get_is_anonymous()
+	       && is_decl(n_type)->get_is_anonymous())
 	{
-	  if (ctxt->get_reporter()->diff_to_be_reported(d.get()))
-	    {
-	      out << indent
-		  << "type of '" << pretty_representation << "' changed:\n";
-	      if (d->currently_reporting())
-		out << indent << "  details are being reported\n";
-	      else if (d->reported_once())
-		out << indent << "  details were reported earlier\n";
-	      else
-		d->report(out, indent + "  ");
-	      begin_with_and = true;
-	    }
+	  out << indent << "while looking at anonymous data member '"
+	      << o_pretty_representation << "':\n"
+	      << indent << "the internal name of that anonymous data member"
+			   " changed from:\n"
+	      << indent << " " << get_type_name(o_type) << "\n"
+	      << indent << "to:\n"
+	      << indent << " " << get_type_name(n_type) << "\n"
+	      << indent << " This is usually due to "
+	      << "an anonymous member type being added or removed from "
+	      << "the containing type\n";
+
+	  begin_with_and = true;
+	  emitted = true;
 	}
     }
+  else if (filtering::has_anonymous_data_member_change(diff))
+    {
+      ABG_ASSERT(o_anon != n_anon);
+      // So we are looking at a non-anonymous data member change from
+      // or to an anonymous data member.
+      const string n_pretty_representation = n->get_pretty_representation();
+      out << indent << (o_anon ? "anonymous " : "")
+	  << "data member " << o_pretty_representation;
+      show_offset_or_size(" at offset", o_offset, *ctxt, out);
+      out << " became " << (n_anon ? "anonymous " : "")
+	  << "data member '" << n_pretty_representation << "'\n";
 
-  if (!filtering::has_anonymous_data_member_change(diff) && name1 != name2)
+      begin_with_and = true;
+      emitted = true;
+    }
+
+  //
+  // If we haven't succeeded in emitting a specific type change report
+  // (mainly related to anonymous data data member changes) then let's
+  // try to emit a more generic report about the type change.
+  //
+  // This is the fallback method outlined in the comment at the
+  // beginning of this section.
+  //
+  if (!emitted)
+    if (const diff_sptr d = diff->type_diff())
+      {
+	if (ctxt->get_reporter()->diff_to_be_reported(d.get()))
+	  {
+	    if (local_only)
+	      out << indent << "type '"
+		  << get_pretty_representation(o->get_type())
+		  << "' of '"
+		  << (o_anon ?
+		      string("anonymous data member")
+		      : o->get_qualified_name())
+		  << "' changed";
+	    else
+	      out << indent
+		  << "type of '"<< (o_anon ? "anonymous data member ": "")
+		  << o_pretty_representation << "' changed";
+
+	    if (d->currently_reporting())
+	      out << ", as being reported\n";
+	    else if (d->reported_once())
+	      out << ", as reported earlier\n";
+	    else
+	      {
+		out << ":\n";
+		d->report(out, indent + "  ");
+	      }
+
+	    begin_with_and = true;
+	    emitted = true;
+	    size_reported = true;
+	  }
+      }
+
+  //
+  // Okay, now we are done with report type changes.  Let's report the
+  // other potential kinds of changes.
+  //
+
+  if (!filtering::has_anonymous_data_member_change(diff) && o_name != n_name)
     {
       if (filtering::has_harmless_name_change(o, n)
 	  && !(ctxt->get_allowed_category()
@@ -532,13 +553,16 @@ represent(const var_diff_sptr	&diff,
 	;
       else
 	{
-	  out << indent;
 	  if (begin_with_and)
 	    {
-	      out << "and ";
+	      out << indent << "and ";
 	      begin_with_and = false;
 	    }
-	  out << "name of '" << name1 << "' changed to '" << name2 << "'";
+	  else if (!emitted)
+	    out << indent;
+	  else
+	    out << ", ";
+	  out << "name of '" << o_name << "' changed to '" << n_name << "'";
 	  report_loc_info(n, *ctxt, out);
 	  emitted = true;
 	}
@@ -553,7 +577,7 @@ represent(const var_diff_sptr	&diff,
 	  begin_with_and = false;
 	}
       else if (!emitted)
-	out << indent << "'" << pretty_representation << "' ";
+	out << indent << "'" << o_pretty_representation << "' ";
       else
 	out << ", ";
       if (get_data_member_is_laid_out(o))
@@ -562,9 +586,9 @@ represent(const var_diff_sptr	&diff,
 	out << "now becomes laid out";
       emitted = true;
     }
-  if ((ctxt->get_allowed_category() & SIZE_OR_OFFSET_CHANGE_CATEGORY))
+  if (show_size_offset_changes)
     {
-      if (get_data_member_offset(o) != get_data_member_offset(n))
+      if (o_offset != n_offset)
 	{
 	  if (begin_with_and)
 	    {
@@ -573,26 +597,20 @@ represent(const var_diff_sptr	&diff,
 	    }
 	  else if (!emitted)
 	    {
+	      out << indent;
 	      if (is_strict_anonymous_data_member_change)
-		out << indent;
-	      else
-		out << indent << "'" << pretty_representation << "' ";
+		out << "anonymous data member ";
+	      out << "'" << o_pretty_representation << "' ";
 	    }
 	  else
 	    out << ", ";
 
-	  show_numerical_change("offset",
-				get_data_member_offset(o),
-				get_data_member_offset(n),
-				*ctxt, out);
+	  show_numerical_change("offset", o_offset, n_offset, *ctxt, out);
 	  maybe_show_relative_offset_change(diff, *ctxt, out);
-
 	  emitted = true;
 	}
-      if (// If we are not displaying only local changes, we must
-	  // have indicated the type size change already.
-	  local_only
-	  && (get_var_size_in_bits(o) != get_var_size_in_bits(n)))
+
+      if (!size_reported && o_size != n_size)
 	{
 	  if (begin_with_and)
 	    {
@@ -601,18 +619,15 @@ represent(const var_diff_sptr	&diff,
 	    }
 	  else if (!emitted)
 	    {
+	      out << indent;
 	      if (is_strict_anonymous_data_member_change)
-		out << indent;
-	      else
-		out << indent << "'" << pretty_representation << "' ";
+		out << "anonymous data member ";
+	      out << "'" << o_pretty_representation << "' ";
 	    }
 	  else
 	    out << ", ";
 
-	  show_numerical_change("size",
-				get_var_size_in_bits(o),
-				get_var_size_in_bits(n),
-				*ctxt, out);
+	  show_numerical_change("size", o_size, n_size, *ctxt, out);
 	  maybe_show_relative_size_change(diff, *ctxt, out);
 	  emitted = true;
 	}
@@ -625,7 +640,7 @@ represent(const var_diff_sptr	&diff,
 	  begin_with_and = false;
 	}
       else if (!emitted)
-	out << indent << "'" << pretty_representation << "' ";
+	out << indent << "'" << o_pretty_representation << "' ";
       else
 	out << ", ";
       out << "elf binding changed from " << o->get_binding()
@@ -640,11 +655,12 @@ represent(const var_diff_sptr	&diff,
 	  begin_with_and = false;
 	}
       else if (!emitted)
-	out << indent << "'" << pretty_representation << "' ";
+	out << indent << "'" << o_pretty_representation << "' ";
       else
 	out << ", ";
       out << "visibility changed from " << o->get_visibility()
 	  << " to " << n->get_visibility();
+      emitted = true;
     }
   if ((ctxt->get_allowed_category() & ACCESS_CHANGE_CATEGORY)
       && (get_member_access_specifier(o)
@@ -656,7 +672,7 @@ represent(const var_diff_sptr	&diff,
 	  begin_with_and = false;
 	}
       else if (!emitted)
-	out << indent << "'" << pretty_representation << "' ";
+	out << indent << "'" << o_pretty_representation << "' ";
       else
 	out << ", ";
 
@@ -675,7 +691,7 @@ represent(const var_diff_sptr	&diff,
 	  begin_with_and = false;
 	}
       else if (!emitted)
-	out << indent << "'" << pretty_representation << "' ";
+	out << indent << "'" << o_pretty_representation << "' ";
       else
 	out << ", ";
 
@@ -683,7 +699,24 @@ represent(const var_diff_sptr	&diff,
 	out << "is no more static";
       else
 	out << "now becomes static";
+      emitted = true;
     }
+
+  if (begin_with_and)
+    // do nothing as begin_with_and implies emitted
+    ;
+  else if (!emitted)
+    // We appear to have fallen off the edge of the map.
+    out << indent << "'" << o_pretty_representation
+	<< "' has *some* difference - please report as a bug";
+  else
+    {
+      ;// do nothing
+    }
+  emitted = true;
+
+  if (!begin_with_and)
+    out << "\n";
 }
 
 /// Report the size and alignment changes of a type.
@@ -697,36 +730,29 @@ represent(const var_diff_sptr	&diff,
 /// @param out the output stream to report the change to.
 ///
 /// @param indent the string to use for indentation.
-///
-/// @param nl whether to start the first report line with a new line.
-///
-/// @return true iff something was reported.
-bool
+void
 report_size_and_alignment_changes(type_or_decl_base_sptr	first,
 				  type_or_decl_base_sptr	second,
 				  diff_context_sptr		ctxt,
 				  ostream&			out,
-				  const string&		indent,
-				  bool				nl)
+				  const string&			indent)
 {
   type_base_sptr f = dynamic_pointer_cast<type_base>(first),
     s = dynamic_pointer_cast<type_base>(second);
 
   if (!s || !f)
-    return false;
+    return;
 
   class_or_union_sptr first_class = is_class_or_union_type(first),
     second_class = is_class_or_union_type(second);
 
-  if (filtering::has_class_decl_only_def_change(first_class, second_class)
-      && !(ctxt->get_allowed_category() & CLASS_DECL_ONLY_DEF_CHANGE_CATEGORY))
+  if (filtering::has_class_decl_only_def_change(first_class, second_class))
     // So these two classes differ only by the fact that one is the
-    // declaration-only form of the second.  And the user asked that
-    // this kind of change be filtered out, so do not report any size
-    // change due to this.
-    return false;
+    // declaration-only form of the second.  The declaration-only class
+    // is of unknown size (recorded as 0) and it is not meaningful to
+    // report a size change.
+    return;
 
-  bool n = false;
   unsigned fs = f->get_size_in_bits(), ss = s->get_size_in_bits(),
     fa = f->get_alignment_in_bits(), sa = s->get_alignment_in_bits();
   array_type_def_sptr first_array = is_array_type(is_type(first)),
@@ -734,9 +760,6 @@ report_size_and_alignment_changes(type_or_decl_base_sptr	first,
   unsigned fdc = first_array ? first_array->get_dimension_count(): 0,
     sdc = second_array ? second_array->get_dimension_count(): 0;
 
-  if (nl)
-    out << "\n";
-
   if (ctxt->get_allowed_category() & SIZE_OR_OFFSET_CHANGE_CATEGORY)
     {
       if (fs != ss || fdc != sdc)
@@ -799,27 +822,23 @@ report_size_and_alignment_changes(type_or_decl_base_sptr	first,
 	    {
 	      out << indent;
 	      show_numerical_change("type size", fs, ss, *ctxt, out);
-	      n = true;
+	      out << "\n";
 	    }
 	} // end if (fs != ss || fdc != sdc)
       else
 	if (ctxt->show_relative_offset_changes())
-	  out << indent << "type size hasn't changed\n";
+	  {
+	    out << indent << "type size hasn't changed\n";
+	  }
     }
   if ((ctxt->get_allowed_category() & SIZE_OR_OFFSET_CHANGE_CATEGORY)
       && (fa != sa))
     {
-      if (n)
-	out << "\n";
       out << indent;
-      show_numerical_change("type alignement", fa, sa, *ctxt, out,
+      show_numerical_change("type alignment", fa, sa, *ctxt, out,
 			    /*show_bits_or_bytes=*/false);
-      n = true;
+      out << "\n";
     }
-
-  if (n)
-    return true;
-  return false;
 }
 
 /// @param tod the type or declaration to emit loc info about
@@ -872,17 +891,12 @@ report_loc_info(const type_or_decl_base_sptr& tod,
 /// @param out the output stream to report the change to.
 ///
 /// @param indent the string to use for indentation.
-///
-/// @param nl whether to start the first report line with a new line.
-///
-/// @return true iff something was reported.
-bool
+void
 report_name_size_and_alignment_changes(decl_base_sptr		first,
 				       decl_base_sptr		second,
 				       diff_context_sptr	ctxt,
-				       ostream&		out,
-				       const string&		indent,
-				       bool			nl)
+				       ostream&			out,
+				       const string&		indent)
 {
   string fn = first->get_qualified_name(),
     sn = second->get_qualified_name();
@@ -896,21 +910,17 @@ report_name_size_and_alignment_changes(decl_base_sptr		first,
 	;
       else
 	{
-	  if (nl)
-	    out << "\n";
 	  out << indent;
 	  if (is_type(first))
 	    out << "type";
 	  else
 	    out << "declaration";
 	  out << " name changed from '" << fn << "' to '" << sn << "'";
-	  nl = true;
+	  out << "\n";
 	}
     }
 
-  nl |= report_size_and_alignment_changes(first, second, ctxt,
-					  out, indent, nl);
-  return nl;
+  report_size_and_alignment_changes(first, second, ctxt, out, indent);
 }
 
 /// Output the header preceding the the report for
@@ -971,7 +981,7 @@ report_mem_header(ostream& out,
 
   if (num_filtered)
     out << " (" << num_filtered << " filtered)";
-  out << colon_or_semi_colon << '\n';
+  out << colon_or_semi_colon << "\n";
 }
 
 /// Output the header preceding the the report for
@@ -1090,7 +1100,7 @@ maybe_report_diff_for_symbol(const elf_symbol_sptr&	symbol1,
 {
   bool reported = false;
 
-  if (!symbol1 ||!symbol2 || symbol1 == symbol2)
+  if (!symbol1 || !symbol2 || symbol1 == symbol2)
     return reported;
 
   if (symbol1->get_size() != symbol2->get_size())
@@ -1180,6 +1190,8 @@ maybe_report_diff_for_symbol(const elf_symbol_sptr&	symbol1,
       out << symbol1->get_version().str()
 	  << " to "
 	  << symbol2->get_version().str();
+
+      reported = true;
     }
 
   if (reported)
@@ -1233,7 +1245,6 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
 				      const string& indent,
 				      ostream& out)
 {
-  const unsigned large_num = 100;
   const diff_context_sptr& ctxt = d.context();
 
   if (!(ctxt->show_unreachable_types()
@@ -1265,19 +1276,13 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
       if (d.priv_->deleted_unreachable_type_is_suppressed((*i).get()))
 	continue;
 
-      out << indent << "  ";
-      if (s.num_removed_unreachable_types() > large_num)
-	out << "[D] ";
-      out << get_pretty_representation(*i);
+      out << indent << "  [D] '" << get_pretty_representation(*i) << "'";
       report_loc_info(*i, *ctxt, out);
       out << "\n";
       emitted = true;
     }
   if (emitted)
-    {
-      out << "\n";
-      emitted = false;
-    }
+    out << "\n";
 
   // Handle changed unreachable types!
   if (s.net_num_changed_unreachable_types() == 1)
@@ -1291,7 +1296,6 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
   diff_sptrs_type sorted_diff_sptrs;
   sort_string_diff_sptr_map(d.priv_->changed_unreachable_types_,
 			    sorted_diff_sptrs);
-  emitted =  true;
   for (diff_sptrs_type::const_iterator i = sorted_diff_sptrs.begin();
        i != sorted_diff_sptrs.end();
        ++i)
@@ -1302,21 +1306,12 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
 
       string repr = diff->first_subject()->get_pretty_representation();
 
-      out << "  ";
-
-      if (sorted_diff_sptrs.size() > large_num)
-	out << "[C] ";
-
-      out << "'" << repr << "' changed:\n";
+      out << indent << "  [C] '" << repr << "' changed:\n";
       diff->report(out, indent + "    ");
+      // Extra spacing.
       out << "\n";
-      emitted = true;
-    }
-  if (emitted)
-    {
-      out << "\n";
-      emitted = false;
     }
+  // Changed types have extra spacing already. No new line here.
 
   // Handle added unreachable types.
   if (s.net_num_added_unreachable_types() == 1)
@@ -1339,19 +1334,13 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
       if (d.priv_->added_unreachable_type_is_suppressed((*i).get()))
 	continue;
 
-      out << indent << "  ";
-      if (s.num_added_unreachable_types() > large_num)
-	out << "[A] ";
-      out << "'" << get_pretty_representation(*i) << "'";
+      out << indent << "  [A] '" << get_pretty_representation(*i) << "'";
       report_loc_info(*i, *ctxt, out);
       out << "\n";
       emitted = true;
     }
   if (emitted)
-    {
-      out << "\n";
-      emitted = false;
-    }
+    out << "\n";
 }
 
 /// If a given diff node impacts some public interfaces, then report
@@ -1362,14 +1351,10 @@ maybe_report_unreachable_type_changes(const corpus_diff& d,
 /// @param out the output stream to report to.
 ///
 /// @param indent the white space string to use for indentation.
-///
-/// @param new_line_prefix if set to true, it means there is going to
-/// be a new line emitted before the report.
 void
 maybe_report_interfaces_impacted_by_diff(const diff	*d,
 					 ostream	&out,
-					 const string	&indent,
-					 bool		new_line_prefix)
+					 const string	&indent)
 {
   const diff_context_sptr &ctxt = d->context();
   const corpus_diff_sptr &corp_diff = ctxt->get_corpus_diff();
@@ -1391,9 +1376,6 @@ maybe_report_interfaces_impacted_by_diff(const diff	*d,
   vector<type_or_decl_base_sptr> sorted_impacted_interfaces;
   sort_artifacts_set(*impacted_artifacts, sorted_impacted_interfaces);
 
-  if (new_line_prefix)
-    out << '\n';
-
   size_t num_impacted_interfaces = impacted_artifacts->size();
   if (num_impacted_interfaces == 1)
     out << indent << "one impacted interface:\n";
@@ -1406,9 +1388,7 @@ maybe_report_interfaces_impacted_by_diff(const diff	*d,
        it != sorted_impacted_interfaces.end();
        ++it)
     {
-      if (it != sorted_impacted_interfaces.begin())
-	out << '\n';
-      out << cur_indent << get_pretty_representation(*it);
+      out << cur_indent << get_pretty_representation(*it) << "\n";
     }
 }
 
@@ -1420,9 +1400,6 @@ maybe_report_interfaces_impacted_by_diff(const diff	*d,
 /// @param out the output stream to report to.
 ///
 /// @param indent the white space string to use for indentation.
-///
-/// @param new_line_prefix if set to true, it means there is going to
-/// be a new line emitted before the report.
 void
 maybe_report_interfaces_impacted_by_diff(const diff_sptr	&d,
 					 ostream		&out,
@@ -1438,5 +1415,81 @@ bool
 reporter_base::diff_to_be_reported(const diff *d) const
 {return d && d->to_be_reported();}
 
-} // namespace comparison
+/// Report about data members replaced by an anonymous data member
+/// without changing the overall bit-layout of the class or union in
+/// an ABI-meaningful way.
+///
+/// @param d the diff to consider.
+///
+/// @param out the output stream to emit the change report to.
+///
+/// @param indent the indentation string to use.
+void
+maybe_report_data_members_replaced_by_anon_dm(const class_or_union_diff &d,
+					      ostream			&out,
+					      const string		indent)
+{
+  const diff_context_sptr& ctxt = d.context();
+
+  if ((ctxt->get_allowed_category() & HARMLESS_DATA_MEMBER_CHANGE_CATEGORY)
+      && !d.data_members_replaced_by_adms().empty())
+    {
+      // Let's detect all the data members that are replaced by
+      // members of the same anonymous data member and report them
+      // in one go.
+      for (changed_var_sptrs_type::const_iterator i =
+	     d.ordered_data_members_replaced_by_adms().begin();
+	   i != d.ordered_data_members_replaced_by_adms().end();)
+	{
+	  // This contains the data members replaced by the same
+	  // anonymous data member.
+	  vector<var_decl_sptr> dms_replaced_by_same_anon_dm;
+	  dms_replaced_by_same_anon_dm.push_back(i->first);
+	  // This contains the anonymous data member that replaced the
+	  // data members in the variable above.
+	  var_decl_sptr anonymous_data_member = i->second;
+	  // Let's look forward to see if the subsequent data
+	  // members were replaced by members of
+	  // anonymous_data_member.
+	  for (++i;
+	       i != d.ordered_data_members_replaced_by_adms().end()
+		 && *i->second == *anonymous_data_member;
+	       ++i)
+	    dms_replaced_by_same_anon_dm.push_back(i->first);
+
+	  bool several_data_members_replaced =
+	    dms_replaced_by_same_anon_dm.size() > 1;
+
+	  out << indent << "data member";
+	  if (several_data_members_replaced)
+	    out << "s";
+
+	  bool first_data_member = true;
+	  for (vector<var_decl_sptr>::const_iterator it =
+		 dms_replaced_by_same_anon_dm.begin();
+	       it != dms_replaced_by_same_anon_dm.end();
+	       ++it)
+	    {
+	      string name = (*it)->get_qualified_name();
+	      if (!first_data_member)
+		out << ",";
+	      out << " '" << name << "'";
+	      first_data_member = false;
+	    }
+
+	  if (several_data_members_replaced)
+	    out << " were ";
+	  else
+	    out << " was ";
+
+	  out << "replaced by anonymous data member:\n"
+	      << indent + "  "
+	      << "'"
+	      << anonymous_data_member->get_pretty_representation()
+	      << "'\n";
+	}
+    }
+}
+
+} // Namespace comparison
 } // end namespace abigail
