@@ -29,8 +29,9 @@
 #define __ABG_SUPPRESSION_PRIV_H__
 
 #include "abg-fwd.h"
-#include "abg-suppression.h"
+#include "abg-regex.h"
 #include "abg-sptr-utils.h"
+#include "abg-suppression.h"
 
 namespace abigail
 {
@@ -47,13 +48,13 @@ class suppression_base::priv
   bool					drops_artifact_;
   string				label_;
   string				file_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	file_name_regex_;
+  mutable regex::regex_t_sptr		file_name_regex_;
   string				file_name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	file_name_not_regex_;
+  mutable regex::regex_t_sptr		file_name_not_regex_;
   string				soname_regex_str_;
-  mutable sptr_utils::regex_t_sptr	soname_regex_;
+  mutable regex::regex_t_sptr		soname_regex_;
   string				soname_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	soname_not_regex_;
+  mutable regex::regex_t_sptr		soname_not_regex_;
 
 public:
   priv()
@@ -87,20 +88,11 @@ public:
   ///
   /// If the 'file_name_regex' property of @ref suppression_base is
   /// empty then this method returns nil.
-  const sptr_utils::regex_t_sptr&
+  const regex::regex_t_sptr&
   get_file_name_regex() const
   {
-    if (!file_name_regex_)
-      {
-	if (!file_name_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			file_name_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      file_name_regex_ = r;
-	  }
-      }
+    if (!file_name_regex_ && !file_name_regex_str_.empty())
+      file_name_regex_ = regex::compile(file_name_regex_str_);
     return file_name_regex_;
   }
 
@@ -112,20 +104,11 @@ public:
   ///
   /// If the 'file_name_not_regex' property of @ref suppression_base
   /// is empty then this method returns nil.
-  const sptr_utils::regex_t_sptr&
+  const regex::regex_t_sptr&
   get_file_name_not_regex() const
   {
-    if (!file_name_not_regex_)
-      {
-	if (!file_name_not_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			file_name_not_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      file_name_not_regex_ = r;
-	  }
-      }
+    if (!file_name_not_regex_ && !file_name_not_regex_str_.empty())
+      file_name_not_regex_ = regex::compile(file_name_not_regex_str_);
     return file_name_not_regex_;
   }
 
@@ -137,20 +120,11 @@ public:
   ///
   /// If the 'soname_regex' property of @ref suppression_base is empty
   /// then this method returns nil.
-  const sptr_utils::regex_t_sptr&
+  const regex::regex_t_sptr&
   get_soname_regex() const
   {
-    if (!soname_regex_)
-      {
-	if (!soname_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			soname_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      soname_regex_ = r;
-	  }
-      }
+    if (!soname_regex_ && !soname_regex_str_.empty())
+      soname_regex_ = regex::compile(soname_regex_str_);
     return soname_regex_;
   }
 
@@ -162,20 +136,11 @@ public:
   ///
   /// If the 'soname_not_regex' property of @ref suppression_base is
   /// empty then this method returns nil.
-  const sptr_utils::regex_t_sptr&
+  const regex::regex_t_sptr&
   get_soname_not_regex() const
   {
-    if (!soname_not_regex_)
-      {
-	if (!soname_not_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			soname_not_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      soname_not_regex_ = r;
-	  }
-      }
+    if (!soname_not_regex_ && !soname_not_regex_str_.empty())
+      soname_not_regex_ = regex::compile(soname_not_regex_str_);
     return soname_not_regex_;
   }
 
@@ -192,17 +157,17 @@ public:
   matches_soname(const string& soname) const
   {
     bool has_regexp = false;
-    if (sptr_utils::regex_t_sptr regexp = get_soname_regex())
+    if (regex::regex_t_sptr regexp = get_soname_regex())
       {
 	has_regexp = true;
-	if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) != 0)
+	if (!regex::match(regexp, soname))
 	  return false;
       }
 
-    if (sptr_utils::regex_t_sptr regexp = get_soname_not_regex())
+    if (regex::regex_t_sptr regexp = get_soname_not_regex())
       {
 	has_regexp = true;
-	if (regexec(regexp.get(), soname.c_str(), 0, NULL, 0) == 0)
+	if (regex::match(regexp, soname))
 	  return false;
       }
 
@@ -227,19 +192,17 @@ public:
   {
     bool has_regexp = false;
 
-    if (sptr_utils::regex_t_sptr regexp = get_file_name_regex())
+    if (regex::regex_t_sptr regexp = get_file_name_regex())
       {
 	has_regexp = true;
-	if (regexec(regexp.get(), binary_name.c_str(),
-	  0, NULL, 0) != 0)
+	if (!regex::match(regexp, binary_name))
 	  return false;
       }
 
-    if (sptr_utils::regex_t_sptr regexp = get_file_name_not_regex())
+    if (regex::regex_t_sptr regexp = get_file_name_not_regex())
       {
 	has_regexp = true;
-	if (regexec(regexp.get(), binary_name.c_str(),
-	  0, NULL, 0) == 0)
+	if (regex::match(regexp, binary_name))
 	  return false;
       }
 
@@ -263,7 +226,7 @@ class function_suppression::parameter_spec::priv
   size_t				index_;
   string				type_name_;
   string				type_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	type_name_regex_;
+  mutable regex::regex_t_sptr		type_name_regex_;
 
   priv()
     : index_()
@@ -277,17 +240,11 @@ class function_suppression::parameter_spec::priv
     : index_(i), type_name_(tn), type_name_regex_str_(tn_regex)
   {}
 
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_type_name_regex() const
   {
     if (!type_name_regex_ && !type_name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    type_name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  type_name_regex_ = r;
-      }
+      type_name_regex_ = regex::compile(type_name_regex_str_);
     return type_name_regex_;
   }
 }; // end class function_suppression::parameter_spec::priv
@@ -302,21 +259,21 @@ struct function_suppression::priv
   change_kind				change_kind_;
   string				name_;
   string				name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	name_regex_;
+  mutable regex::regex_t_sptr		name_regex_;
   string				name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	name_not_regex_;
+  mutable regex::regex_t_sptr		name_not_regex_;
   string				return_type_name_;
   string				return_type_regex_str_;
-  mutable sptr_utils::regex_t_sptr	return_type_regex_;
+  mutable regex::regex_t_sptr		return_type_regex_;
   parameter_specs_type			parm_specs_;
   string				symbol_name_;
   string				symbol_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_name_regex_;
+  mutable regex::regex_t_sptr		symbol_name_regex_;
   string				symbol_name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_name_not_regex_;
+  mutable regex::regex_t_sptr		symbol_name_not_regex_;
   string				symbol_version_;
   string				symbol_version_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_version_regex_;
+  mutable regex::regex_t_sptr		symbol_version_regex_;
   bool					allow_other_aliases_;
 
   priv():
@@ -356,17 +313,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::name_regex_str_..
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_name_regex() const
   {
     if (!name_regex_ && !name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  name_regex_ = r;
-      }
+      name_regex_ = regex::compile(name_regex_str_);
     return name_regex_;
   }
 
@@ -379,17 +330,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::name_not_regex_str_..
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_name_not_regex() const
   {
     if (!name_not_regex_ && !name_not_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    name_not_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  name_not_regex_ = r;
-      }
+      name_not_regex_ = regex::compile(name_not_regex_str_);
     return name_not_regex_;
   }
 
@@ -402,17 +347,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::return_type_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_return_type_regex() const
   {
     if (!return_type_regex_ && !return_type_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    return_type_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  return_type_regex_ = r;
-      }
+      return_type_regex_ = regex::compile(return_type_regex_str_);
     return return_type_regex_;
   }
 
@@ -425,17 +364,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::symbol_name_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_name_regex() const
   {
     if (!symbol_name_regex_ && !symbol_name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    symbol_name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  symbol_name_regex_ = r;
-      }
+      symbol_name_regex_ = regex::compile(symbol_name_regex_str_);
     return symbol_name_regex_;
   }
 
@@ -448,17 +381,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::symbol_name_not_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_name_not_regex() const
   {
     if (!symbol_name_not_regex_ && !symbol_name_not_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    symbol_name_not_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  symbol_name_not_regex_ = r;
-      }
+      symbol_name_not_regex_ = regex::compile(symbol_name_not_regex_str_);
     return symbol_name_not_regex_;
   }
 
@@ -471,17 +398,11 @@ struct function_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// function_suppression::priv::symbol_version_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_version_regex() const
   {
-    if (!symbol_version_regex_ && ! symbol_version_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    symbol_version_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  symbol_version_regex_ = r;
-      }
+    if (!symbol_version_regex_ && !symbol_version_regex_str_.empty())
+      symbol_version_regex_ = regex::compile(symbol_version_regex_str_);
     return symbol_version_regex_;
   }
 }; // end class function_suppression::priv
@@ -561,20 +482,20 @@ struct variable_suppression::priv
   change_kind				change_kind_;
   string				name_;
   string				name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	name_regex_;
+  mutable regex::regex_t_sptr		name_regex_;
   string				name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	name_not_regex_;
+  mutable regex::regex_t_sptr		name_not_regex_;
   string				symbol_name_;
   string				symbol_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_name_regex_;
+  mutable regex::regex_t_sptr		symbol_name_regex_;
   string				symbol_name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_name_not_regex_;
+  mutable regex::regex_t_sptr		symbol_name_not_regex_;
   string				symbol_version_;
   string				symbol_version_regex_str_;
-  mutable sptr_utils::regex_t_sptr	symbol_version_regex_;
+  mutable regex::regex_t_sptr		symbol_version_regex_;
   string				type_name_;
   string				type_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	type_name_regex_;
+  mutable regex::regex_t_sptr		type_name_regex_;
 
   priv(const string& name,
        const string& name_regex_str,
@@ -604,17 +525,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::name_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_name_regex() const
   {
     if (!name_regex_ && !name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  name_regex_ = r;
-      }
+      name_regex_ = regex::compile(name_regex_str_);
     return name_regex_;
   }
 
@@ -627,17 +542,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::name_not_regex_str_..
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_name_not_regex() const
   {
     if (!name_not_regex_ && !name_not_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    name_not_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  name_not_regex_ = r;
-      }
+      name_not_regex_ = regex::compile(name_not_regex_str_);
     return name_not_regex_;
   }
 
@@ -650,17 +559,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::symbol_name_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_name_regex() const
   {
     if (!symbol_name_regex_ && !symbol_name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    symbol_name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  symbol_name_regex_ = r;
-      }
+      symbol_name_regex_ = regex::compile(symbol_name_regex_str_);
     return symbol_name_regex_;
   }
 
@@ -673,16 +576,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::symbol_name_not_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_name_not_regex() const
   {
     if (!symbol_name_not_regex_ && !symbol_name_not_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(), symbol_name_not_regex_str_.c_str(),
-		    REG_EXTENDED == 0) == 0)
-	  symbol_name_not_regex_ = r;
-      }
+      symbol_name_not_regex_ = regex::compile(symbol_name_not_regex_str_);
     return symbol_name_not_regex_;
   }
 
@@ -695,17 +593,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::symbol_version_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_symbol_version_regex()  const
   {
     if (!symbol_version_regex_ && !symbol_version_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    symbol_version_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  symbol_version_regex_ = r;
-      }
+      symbol_version_regex_ = regex::compile(symbol_version_regex_str_);
     return symbol_version_regex_;
   }
 
@@ -718,17 +610,11 @@ struct variable_suppression::priv
   ///
   /// @return a pointer to the regular expression object of
   /// variable_suppression::priv::type_name_regex_str_.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_type_name_regex() const
   {
     if (!type_name_regex_ && !type_name_regex_str_.empty())
-      {
-	sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	if (regcomp(r.get(),
-		    type_name_regex_str_.c_str(),
-		    REG_EXTENDED) == 0)
-	  type_name_regex_ = r;
-      }
+      type_name_regex_ = regex::compile(type_name_regex_str_);
     return type_name_regex_;
   }
 };// end class variable_supppression::priv
@@ -766,18 +652,18 @@ variable_is_suppressed(const ReadContextType&	ctxt,
 class type_suppression::priv
 {
   string				type_name_regex_str_;
-  mutable sptr_utils::regex_t_sptr	type_name_regex_;
+  mutable regex::regex_t_sptr		type_name_regex_;
   string				type_name_;
   string				type_name_not_regex_str_;
-  mutable sptr_utils::regex_t_sptr	type_name_not_regex_;
+  mutable regex::regex_t_sptr		type_name_not_regex_;
   bool					consider_type_kind_;
   type_suppression::type_kind		type_kind_;
   bool					consider_reach_kind_;
   type_suppression::reach_kind		reach_kind_;
   type_suppression::insertion_ranges	insertion_ranges_;
-  unordered_set<string>		source_locations_to_keep_;
+  unordered_set<string>			source_locations_to_keep_;
   string				source_location_to_keep_regex_str_;
-  mutable sptr_utils::regex_t_sptr	source_location_to_keep_regex_;
+  mutable regex::regex_t_sptr		source_location_to_keep_regex_;
   mutable vector<string>		changed_enumerator_names_;
 
   priv();
@@ -805,20 +691,11 @@ public:
   ///
   /// If the 'type_name_regex' property of @ref type_suppression is
   /// empty then this method returns nil.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_type_name_regex() const
   {
-    if (!type_name_regex_)
-      {
-	if (!type_name_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			type_name_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      type_name_regex_ = r;
-	  }
-      }
+    if (!type_name_regex_ && !type_name_regex_str_.empty())
+      type_name_regex_ = regex::compile(type_name_regex_str_);
     return type_name_regex_;
   }
 
@@ -826,7 +703,7 @@ public:
   ///
   /// @param r the new type_name_regex object.
   void
-  set_type_name_regex(sptr_utils::regex_t_sptr r)
+  set_type_name_regex(regex::regex_t_sptr r)
   {type_name_regex_ = r;}
 
   /// Get the regular expression object associated to the
@@ -837,20 +714,11 @@ public:
   ///
   /// If the 'type_name_not_regex' property of @ref type_suppression is
   /// empty then this method returns nil.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_type_name_not_regex() const
   {
-    if (!type_name_not_regex_)
-      {
-	if (!type_name_not_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			type_name_not_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      type_name_not_regex_ = r;
-	  }
-      }
+    if (!type_name_not_regex_ && !type_name_not_regex_str_.empty())
+      type_name_not_regex_ = regex::compile(type_name_not_regex_str_);
     return type_name_not_regex_;
   }
 
@@ -858,7 +726,7 @@ public:
   ///
   /// @param r the new type_name_not_regex object.
   void
-  set_type_name_not_regex(sptr_utils::regex_t_sptr r)
+  set_type_name_not_regex(regex::regex_t_sptr r)
   {type_name_not_regex_ = r;}
 
   /// Getter for the string that denotes the 'type_name_not_regex'
@@ -882,20 +750,13 @@ public:
   /// Getter for the source_location_to_keep_regex object.
   ///
   /// This function builds the regex if it's not yet built.
-  const sptr_utils::regex_t_sptr
+  const regex::regex_t_sptr
   get_source_location_to_keep_regex() const
   {
-    if (!source_location_to_keep_regex_)
-      {
-	if (!source_location_to_keep_regex_str_.empty())
-	  {
-	    sptr_utils::regex_t_sptr r = sptr_utils::build_sptr<regex_t>();
-	    if (regcomp(r.get(),
-			source_location_to_keep_regex_str_.c_str(),
-			REG_EXTENDED) == 0)
-	      source_location_to_keep_regex_ = r;
-	  }
-      }
+    if (!source_location_to_keep_regex_
+	&& !source_location_to_keep_regex_str_.empty())
+      source_location_to_keep_regex_ =
+	  regex::compile(source_location_to_keep_regex_str_);
     return source_location_to_keep_regex_;
   }
 
@@ -903,7 +764,7 @@ public:
   ///
   /// @param r the new regex object.
   void
-  set_source_location_to_keep_regex(sptr_utils::regex_t_sptr r)
+  set_source_location_to_keep_regex(regex::regex_t_sptr r)
   {source_location_to_keep_regex_ = r;}
 
   friend class type_suppression;
