@@ -61,6 +61,8 @@
 #include "abg-dwarf-reader.h"
 #include "abg-internal.h"
 #include "abg-cxx-compat.h"
+#include "abg-regex.h"
+
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
 
@@ -508,61 +510,45 @@ get_anonymous_enum_internal_name_prefix()
 bool
 decl_names_equal(const string& l, const string& r)
 {
-  string::size_type l_pos1 = 0, l_pos2 = 0, r_pos1 = 0, r_pos2 = 0;
-  string::size_type l_length = l.length(), r_length = r.length();
+  string::size_type l_pos1 = 0, r_pos1 = 0;
+  const string::size_type l_length = l.length(), r_length = r.length();
 
   while (l_pos1 < l_length && r_pos1 < r_length)
     {
-      l_pos2 = l.find("::", l_pos1);
-      r_pos2 = r.find("::", r_pos1);
-
-      if ((l.compare(l_pos1,
-		     ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
-		     ANONYMOUS_STRUCT_INTERNAL_NAME) == 0
-	   && r.compare(r_pos1,
-			ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
-			ANONYMOUS_STRUCT_INTERNAL_NAME) == 0)
-	  ||
-	  (l.compare(l_pos1,
-		     ANONYMOUS_UNION_INTERNAL_NAME_LEN,
-		     ANONYMOUS_UNION_INTERNAL_NAME) == 0
-	   && r.compare(r_pos1,
-			ANONYMOUS_UNION_INTERNAL_NAME_LEN,
-			ANONYMOUS_UNION_INTERNAL_NAME) == 0)
-	  ||
-	  (l.compare(l_pos1,
-		     ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
-		     ANONYMOUS_ENUM_INTERNAL_NAME) == 0
-	   && r.compare(r_pos1,
-			ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
-			ANONYMOUS_ENUM_INTERNAL_NAME) == 0))
-	{
-	  if (l_pos2 == l.npos || r_pos2 == r.npos)
-	    return true;
-
-	  l_pos1 = l_pos2 + 2;
-	  r_pos1 = r_pos2 + 2;
-	  continue;
-	}
-
-      if (l_pos2 == l.npos || r_pos2 == r.npos)
-	{
-	  if (l_pos2 != r_pos2)
-	    return false;
-
-	  return !l.compare(l_pos1, l_pos2, r,
-			    r_pos1, r_pos2);
-	}
+      string::size_type l_pos2 = l.find("::", l_pos1);
+      string::size_type r_pos2 = r.find("::", r_pos1);
+      if (l_pos2 == string::npos)
+	l_pos2 = l_length;
+      if (r_pos2 == string::npos)
+	r_pos2 = r_length;
 
       if (l.compare(l_pos1, l_pos2 - l_pos1, r,
-		    r_pos1, r_pos2 - r_pos1))
+		    r_pos1, r_pos2 - r_pos1)
+	  && (l.compare(l_pos1,
+			ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
+			ANONYMOUS_STRUCT_INTERNAL_NAME)
+	      || r.compare(r_pos1,
+			   ANONYMOUS_STRUCT_INTERNAL_NAME_LEN,
+			   ANONYMOUS_STRUCT_INTERNAL_NAME))
+	  && (l.compare(l_pos1,
+			ANONYMOUS_UNION_INTERNAL_NAME_LEN,
+			ANONYMOUS_UNION_INTERNAL_NAME)
+	      || r.compare(r_pos1,
+			   ANONYMOUS_UNION_INTERNAL_NAME_LEN,
+			   ANONYMOUS_UNION_INTERNAL_NAME))
+	  && (l.compare(l_pos1,
+			ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
+			ANONYMOUS_ENUM_INTERNAL_NAME)
+	      || r.compare(r_pos1,
+			   ANONYMOUS_ENUM_INTERNAL_NAME_LEN,
+			   ANONYMOUS_ENUM_INTERNAL_NAME)))
 	return false;
 
-      l_pos1 = l_pos2 + 2;
-      r_pos1 = r_pos2 + 2;
+      l_pos1 = l_pos2 == l_length ? l_pos2 : l_pos2 + 2;
+      r_pos1 = r_pos2 == r_length ? r_pos2 : r_pos2 + 2;
     }
 
-  return true;
+  return (l_pos1 == l_length) == (r_pos1 == r_length);
 }
 
 /// If a given file is a symbolic link, get the canonicalized absolute
@@ -867,7 +853,7 @@ string_is_ascii(const string& str)
 ///
 /// "if the hexadecimal value for a universal-character-name [...]  or
 ///  string literal corresponds to a control character (in either of
-///  the ranges 0x00–0x1F or 0x7F–0x9F, both inclusive) [...] the
+///  the ranges 0x00-0x1F or 0x7F-0x9F, both inclusive) [...] the
 ///  program is ill-formed."
 ///
 /// @param str the string to consider.
@@ -1787,6 +1773,43 @@ make_path_absolute_to_be_freed(const char*p)
   return result;
 }
 
+/// This is a sub-routine of gen_suppr_spec_from_headers and
+/// handle_fts_entry.
+///
+/// It setups a type suppression which is meant to keep types defined
+/// in a given file and suppress all other types.
+///
+/// @param file_path the path to the file that defines types that are
+/// meant to be kept by the type suppression.  All other types defined
+/// in other files are to be suppressed.  Note that this file path is
+/// added to the vector returned by
+/// type_suppression::get_source_locations_to_keep()
+///
+/// @param suppr the type suppression to setup.  If this smart pointer
+/// is nil then a new instance @ref type_suppression is created and
+/// this variable is made to point to it.
+static void
+handle_file_entry(const string& file_path,
+		  type_suppression_sptr& suppr)
+{
+  if (!suppr)
+    {
+      suppr.reset(new type_suppression(get_private_types_suppr_spec_label(),
+				       /*type_name_regexp=*/"",
+				       /*type_name=*/""));
+
+      // Types that are defined in system headers are usually
+      // OK to be considered as public types.
+      suppr->set_source_location_to_keep_regex_str("^/usr/include/");
+      suppr->set_is_artificial(true);
+    }
+
+  // And types that are defined in header files that are under
+  // the header directory file we are looking are to be
+  // considered public types too.
+  suppr->get_source_locations_to_keep().insert(file_path);
+}
+
 /// This is a sub-routine of gen_suppr_spec_from_headers.
 ///
 /// @param entry if this file represents a regular (or symlink) file,
@@ -1815,28 +1838,102 @@ handle_fts_entry(const FTSENT *entry,
       if (string_ends_with(fname, ".h")
 	  || string_ends_with(fname, ".hpp")
 	  || string_ends_with(fname, ".hxx"))
-	{
-	  if (!suppr)
-	    {
-	      suppr.reset(new type_suppression(get_private_types_suppr_spec_label(),
-					       /*type_name_regexp=*/"",
-					       /*type_name=*/""));
+	handle_file_entry (fname, suppr);
+    }
+}
 
-	      // Types that are defined in system headers are usually
-	      // OK to be considered as public types.
-	      suppr->set_source_location_to_keep_regex_str("^/usr/include/");
-	      suppr->set_is_artificial(true);
-	    }
-	  // And types that are defined in header files that are under
-	  // the header directory file we are looking are to be
-	  // considered public types too.
-	  suppr->get_source_locations_to_keep().insert(fname);
+/// Populate a type_supression from header files found in a given
+/// directory tree.
+///
+/// The suppression suppresses types defined in source files that are
+/// *NOT* found in the directory tree.
+///
+/// This is a subroutine for gen_suppr_spect_from_headers.
+///
+/// @param headers_root_dir the directory tree to consider for header
+/// files.
+///
+/// @param result the type_supression to populate from the content of
+/// @p headers_root_dir.
+static void
+gen_suppr_spec_from_headers_root_dir(const string& headers_root_dir,
+				     type_suppression_sptr &result)
+{
+  if (!headers_root_dir.empty())
+    {
+      char* paths[] = {const_cast<char*>(headers_root_dir.c_str()), 0};
+
+      if (FTS *file_hierarchy = fts_open(paths, FTS_LOGICAL|FTS_NOCHDIR, NULL))
+	{
+	  FTSENT *entry;
+	  while ((entry = fts_read(file_hierarchy)))
+	    handle_fts_entry(entry, result);
+	  fts_close(file_hierarchy);
 	}
     }
 }
 
 /// Generate a type suppression specification that suppresses ABI
-/// changes for types defines in source files that are *NOT* in a give
+/// changes for types defined in source files that are neither in a
+/// given set of header root directories nor in a set of header
+/// files.
+///
+/// @param headers_root_dirs ABI changes in types defined in files
+/// *NOT* found in these directory trees are going be suppressed.
+///
+/// @param header_files a set of additional header files that define
+/// types that are to be kept (not supressed) by the returned type
+/// suppression.
+///
+/// @return the resulting type suppression generated, if any file was
+/// found in the directory tree @p headers_root_dir.
+type_suppression_sptr
+gen_suppr_spec_from_headers(const vector<string>& headers_root_dirs,
+			    const vector<string>& header_files)
+{
+  type_suppression_sptr result;
+
+  for (vector<string>::const_iterator root_dir = headers_root_dirs.begin();
+       root_dir != headers_root_dirs.end();
+       ++root_dir)
+    gen_suppr_spec_from_headers_root_dir(*root_dir, result);
+
+  for (vector<string>::const_iterator file = header_files.begin();
+       file != header_files.end();
+       ++file)
+    handle_file_entry(*file, result);
+
+  return result;
+}
+
+/// Generate a type suppression specification that suppresses ABI
+/// changes for types defined in source files that are neither in a
+/// given header root dir, not in a set of header files.
+///
+/// @param headers_root_dir ABI changes in types defined in files
+/// *NOT* found in this directory tree are going be suppressed.
+///
+/// @param header_files a set of additional header files that define
+/// types that are to be kept (not supressed) by the returned type
+/// suppression.
+///
+/// @return the resulting type suppression generated, if any file was
+/// found in the directory tree @p headers_root_dir.
+type_suppression_sptr
+gen_suppr_spec_from_headers(const string& headers_root_dir,
+			    const vector<string>& header_files)
+{
+  type_suppression_sptr result;
+  vector<string> root_dirs;
+
+  if (!headers_root_dir.empty())
+    root_dirs.push_back(headers_root_dir);
+
+  return gen_suppr_spec_from_headers(root_dirs, header_files);
+}
+
+/// Generate a type suppression specification that suppresses ABI
+/// changes for types defined in source files that are not in a given
 /// header root dir.
 ///
 /// @param headers_root_dir ABI changes in types defined in files
@@ -1847,24 +1944,9 @@ handle_fts_entry(const FTSENT *entry,
 type_suppression_sptr
 gen_suppr_spec_from_headers(const string& headers_root_dir)
 {
-  type_suppression_sptr result;
-
-  if (headers_root_dir.empty())
-    // We were given no headers root dir so the resulting suppression
-    // specification shall be empty.
-    return result;
-
-  char* paths[] = {const_cast<char*>(headers_root_dir.c_str()), 0};
-
-  FTS *file_hierarchy = fts_open(paths, FTS_LOGICAL|FTS_NOCHDIR, NULL);
-  if (!file_hierarchy)
-    return result;
-
-  FTSENT *entry;
-  while ((entry = fts_read(file_hierarchy)))
-    handle_fts_entry(entry, result);
-  fts_close(file_hierarchy);
-  return result;
+  // We don't list individual files, just look under the headers_path.
+  vector<string> header_files;
+  return gen_suppr_spec_from_headers(headers_root_dir, header_files);
 }
 
 /// Generate a suppression specification from kernel abi whitelist
@@ -1950,13 +2032,7 @@ gen_suppr_spec_from_kernel_abi_whitelists
 
       // Build a regular expression representing the union of all
       // the function and variable names expressed in the white list.
-      std::stringstream regex_ss;
-      regex_ss << "^";
-      std::copy(whitelisted_names.begin(), whitelisted_names.end(),
-		std::ostream_iterator<std::string>(regex_ss, "$|^"));
-      regex_ss.seekp(0, std::ios::end);
-      const std::string& regex =
-	  regex_ss.str().substr(0, static_cast<size_t>(regex_ss.tellp()) - 2);
+      const std::string regex = regex::generate_from_strings(whitelisted_names);
 
       // Build a suppression specification which *keeps* functions
       // whose ELF symbols match the regular expression contained
@@ -2382,17 +2458,15 @@ get_vmlinux_path_from_kernel_dist(const string&	from,
     if (!dir_exists(from))
     return false;
 
-  // For now, we assume either an Enterprise Linux or a Fedora kernel
-  // distribution directory.
-  //
-  // We also take into account split debug info package for these.  In
-  // this case, the content split debug info package is installed
-  // under the 'dist_root' directory as well, and its content is
-  // accessible from <dist_root>/usr/lib/debug directory.
+  // For now, we assume the possibility of having either an Enterprise
+  // Linux or a Fedora kernel distribution directory.  In those cases,
+  // the vmlinux binary is located under the /lib/modules
+  // sub-directory.  So we might as well save some time by picking it
+  // from there directly.
 
     string dist_root = from;
   if (dir_exists(dist_root + "/lib/modules"))
-    dist_root + "/lib/modules";
+    dist_root += "/lib/modules";
 
   bool found = false;
   if (find_vmlinux_path(dist_root, vmlinux_path))
