@@ -619,6 +619,34 @@ static InOutSpec in_out_specs[] =
     "data/test-diff-pkg/PR24690/PR24690-report-0.txt",
     "output/test-diff-pkg/PR24690/PR24690-report-0.txt"
   },
+#if WITH_RPM_4_15
+  // This RPM version supports packages compressed with zstd.
+  // RPMs from Fedora 31 onwards are compressed with zstd.
+  {
+    "data/test-diff-pkg/mesa-libGLU-9.0.1-3.fc33.x86_64.rpm",
+    "data/test-diff-pkg/mesa-libGLU-9.0.1-3.fc33.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/mesa-libGLU-debuginfo-9.0.1-3.fc33.x86_64.rpm",
+    "data/test-diff-pkg/mesa-libGLU-debuginfo-9.0.1-3.fc33.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/mesa-libGLU-9.0.1-3.fc33.x86_64.self-check-report-0.txt",
+    "output/test-diff-pkg/mesa-libGLU-9.0.1-3.fc33.x86_64.self-check-report-0.txt"
+  },
+  {
+    "data/test-diff-pkg/hdf5-1.10.6-2.fc33.x86_64.rpm",
+    "data/test-diff-pkg/hdf5-1.10.6-2.fc33.x86_64.rpm",
+    "--self-check",
+    "",
+    "data/test-diff-pkg/hdf5-debuginfo-1.10.6-2.fc33.x86_64.rpm",
+    "data/test-diff-pkg/hdf5-debuginfo-1.10.6-2.fc33.x86_64.rpm",
+    "",
+    "",
+    "data/test-diff-pkg/hdf5-1.10.6-2.fc33.x86_64.self-check-report-0.txt",
+    "output/test-diff-pkg/hdf5-1.10.6-2.fc33.x86_64.self-check-report-0.txt"
+  },
+#endif // WITH_RPM_4_15
 #endif //WITH_RPM
 
 #ifdef WITH_DEB
@@ -840,7 +868,8 @@ main()
 	{
 	  is_ok = false;
 	  if (!t->diff_cmd.empty())
-	    system(t->diff_cmd.c_str());
+	    if (system(t->diff_cmd.c_str()) == -1)
+	      cerr << "execution of '" << t->diff_cmd << "' failed\n";
 	  if (!t->error_message.empty())
 	    cerr << t->error_message << '\n';
 	}
