@@ -640,8 +640,9 @@ SSL_CanBypass(CERTCertificate *cert, SECKEYPrivateKey *srvPrivkey,
      * SSL3 and TLS specs forbid negotiating that cipher suite number.
      */
     for (i = 0; i < nsuites && (suite = *ciphersuites++) != 0; i++) {
-        /* skip cipher suites NSS doesn't support */
-        if (SSL_GetCipherSuiteInfo(suite, &csdef, sizeof(csdef)) != SECSuccess)
+        /* skip SSL2 cipher suites and ones NSS doesn't support */
+        if (SSL_GetCipherSuiteInfo(suite, &csdef, sizeof(csdef)) != SECSuccess ||
+            SSL_IS_SSL2_CIPHER(suite))
             continue;
         switch (csdef.keaType) {
             case ssl_kea_rsa:
