@@ -276,17 +276,6 @@ static void vmsd_desc_field_end(const VMStateDescription *vmsd, QJSON *vmdesc,
     json_end_object(vmdesc);
 }
 
-
-bool vmstate_save_needed(const VMStateDescription *vmsd, void *opaque)
-{
-    if (vmsd->needed && !vmsd->needed(opaque)) {
-        /* optional section not needed */
-        return false;
-    }
-    return true;
-}
-
-
 void vmstate_save_state(QEMUFile *f, const VMStateDescription *vmsd,
                         void *opaque, QJSON *vmdesc)
 {
@@ -352,11 +341,11 @@ void vmstate_save_state(QEMUFile *f, const VMStateDescription *vmsd,
 }
 
 static const VMStateDescription *
-vmstate_get_subsection(const VMStateDescription **sub, char *idstr)
+    vmstate_get_subsection(const VMStateSubsection *sub, char *idstr)
 {
-    while (sub && *sub && (*sub)->needed) {
-        if (strcmp(idstr, (*sub)->name) == 0) {
-            return *sub;
+    while (sub && sub->needed) {
+        if (strcmp(idstr, sub->vmsd->name) == 0) {
+            return sub->vmsd;
         }
         sub++;
     }
@@ -416,12 +405,12 @@ static int vmstate_subsection_load(QEMUFile *f, const VMStateDescription *vmsd,
 static void vmstate_subsection_save(QEMUFile *f, const VMStateDescription *vmsd,
                                     void *opaque, QJSON *vmdesc)
 {
-    const VMStateDescription **sub = vmsd->subsections;
+    const VMStateSubsection *sub = vmsd->subsections;
     bool subsection_found = false;
 
-    while (sub && *sub && (*sub)->needed) {
-        if ((*sub)->needed(opaque)) {
-            const VMStateDescription *vmsd = *sub;
+    while (sub && sub->needed) {
+        if (sub->needed(opaque)) {
+            const VMStateDescription *vmsd = sub->vmsd;
             uint8_t len;
 
             if (vmdesc) {
