@@ -22,6 +22,7 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <netinet/in.h>
@@ -30,6 +31,9 @@
 #include <unistd.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#ifdef HAVE_LINUX_SOCKIOS_H
+#include <linux/sockios.h> /* SIOCOUTQ */
+#endif
 
 #include "common/generated_server_marshallers.h"
 #include "common/ring.h"
@@ -37,6 +41,7 @@
 #include "stat.h"
 #include "red_channel.h"
 #include "reds.h"
+#include "reds_stream.h"
 #include "main_dispatcher.h"
 #include "red_time.h"
 
@@ -116,47 +121,47 @@ static void red_channel_client_unref(RedChannelClient *rcc);
 
 static uint32_t full_header_get_msg_size(SpiceDataHeaderOpaque *header)
 {
-    return ((SpiceDataHeader *)header->data)->size;
+    return GUINT32_FROM_LE(((SpiceDataHeader *)header->data)->size);
 }
 
 static uint32_t mini_header_get_msg_size(SpiceDataHeaderOpaque *header)
 {
-    return ((SpiceMiniDataHeader *)header->data)->size;
+    return GUINT32_FROM_LE(((SpiceMiniDataHeader *)header->data)->size);
 }
 
 static uint16_t full_header_get_msg_type(SpiceDataHeaderOpaque *header)
 {
-    return ((SpiceDataHeader *)header->data)->type;
+    return GUINT16_FROM_LE(((SpiceDataHeader *)header->data)->type);
 }
 
 static uint16_t mini_header_get_msg_type(SpiceDataHeaderOpaque *header)
 {
-    return ((SpiceMiniDataHeader *)header->data)->type;
+    return GUINT16_FROM_LE(((SpiceMiniDataHeader *)header->data)->type);
 }
 
 static void full_header_set_msg_type(SpiceDataHeaderOpaque *header, uint16_t type)
 {
-    ((SpiceDataHeader *)header->data)->type = type;
+    ((SpiceDataHeader *)header->data)->type = GUINT16_TO_LE(type);
 }
 
 static void mini_header_set_msg_type(SpiceDataHeaderOpaque *header, uint16_t type)
 {
-    ((SpiceMiniDataHeader *)header->data)->type = type;
+    ((SpiceMiniDataHeader *)header->data)->type = GUINT16_TO_LE(type);
 }
 
 static void full_header_set_msg_size(SpiceDataHeaderOpaque *header, uint32_t size)
 {
-    ((SpiceDataHeader *)header->data)->size = size;
+    ((SpiceDataHeader *)header->data)->size = GUINT32_TO_LE(size);
 }
 
 static void mini_header_set_msg_size(SpiceDataHeaderOpaque *header, uint32_t size)
 {
-    ((SpiceMiniDataHeader *)header->data)->size = size;
+    ((SpiceMiniDataHeader *)header->data)->size = GUINT32_TO_LE(size);
 }
 
 static void full_header_set_msg_serial(SpiceDataHeaderOpaque *header, uint64_t serial)
 {
-    ((SpiceDataHeader *)header->data)->serial = serial;
+    ((SpiceDataHeader *)header->data)->serial = GUINT64_TO_LE(serial);
 }
 
 static void mini_header_set_msg_serial(SpiceDataHeaderOpaque *header, uint64_t serial)
@@ -166,7 +171,7 @@ static void mini_header_set_msg_serial(SpiceDataHeaderOpaque *header, uint64_t s
 
 static void full_header_set_msg_sub_list(SpiceDataHeaderOpaque *header, uint32_t sub_list)
 {
-    ((SpiceDataHeader *)header->data)->sub_list = sub_list;
+    ((SpiceDataHeader *)header->data)->sub_list = GUINT32_TO_LE(sub_list);
 }
 
 static void mini_header_set_msg_sub_list(SpiceDataHeaderOpaque *header, uint32_t sub_list)
@@ -450,7 +455,7 @@ static void red_channel_client_reset_send_data(RedChannelClient *rcc)
     rcc->send_data.header.set_msg_type(&rcc->send_data.header, 0);
     rcc->send_data.header.set_msg_size(&rcc->send_data.header, 0);
 
-    /* Keeping the serial consecutive: reseting it if reset_send_data
+    /* Keeping the serial consecutive: resetting it if reset_send_data
      * has been called before, but no message has been sent since then.
      */
     if (rcc->send_data.last_sent_serial != rcc->send_data.serial) {
@@ -534,7 +539,7 @@ static void red_channel_client_send_ping(RedChannelClient *rcc)
         /*
          * When testing latency, TCP_NODELAY must be switched on, otherwise,
          * sending the ping message is delayed by Nagle algorithm, and the
-         * roundtrip measurment is less accurate (bigger).
+         * roundtrip measurement is less accurate (bigger).
          */
         rcc->latency_monitor.tcp_nodelay = 1;
         if (getsockopt(rcc->stream->socket, IPPROTO_TCP, TCP_NODELAY, &delay_val,
@@ -595,8 +600,6 @@ static void red_channel_client_send_item(RedChannelClient *rcc, PipeItem *item)
 
 static void red_channel_client_release_item(RedChannelClient *rcc, PipeItem *item, int item_pushed)
 {
-    int handled = TRUE;
-
     switch (item->type) {
         case PIPE_ITEM_TYPE_SET_ACK:
         case PIPE_ITEM_TYPE_EMPTY_MSG:
@@ -605,10 +608,7 @@ static void red_channel_client_release_item(RedChannelClient *rcc, PipeItem *ite
             free(item);
             break;
         default:
-            handled = FALSE;
-    }
-    if (!handled) {
-        rcc->channel->channel_cbs.release_item(rcc, item, item_pushed);
+            rcc->channel->channel_cbs.release_item(rcc, item, item_pushed);
     }
 }
 
@@ -680,16 +680,16 @@ static void red_channel_client_destroy_remote_caps(RedChannelClient* rcc)
 
 int red_channel_client_test_remote_common_cap(RedChannelClient *rcc, uint32_t cap)
 {
-    return test_capabilty(rcc->remote_caps.common_caps,
-                          rcc->remote_caps.num_common_caps,
-                          cap);
+    return test_capability(rcc->remote_caps.common_caps,
+                           rcc->remote_caps.num_common_caps,
+                           cap);
 }
 
 int red_channel_client_test_remote_cap(RedChannelClient *rcc, uint32_t cap)
 {
-    return test_capabilty(rcc->remote_caps.caps,
-                          rcc->remote_caps.num_caps,
-                          cap);
+    return test_capability(rcc->remote_caps.caps,
+                           rcc->remote_caps.num_caps,
+                           cap);
 }
 
 int red_channel_test_remote_common_cap(RedChannel *channel, uint32_t cap)
@@ -742,21 +742,30 @@ static void red_channel_client_push_ping(RedChannelClient *rcc)
 
 static void red_channel_client_ping_timer(void *opaque)
 {
-    int so_unsent_size = 0;
     RedChannelClient *rcc = opaque;
 
     spice_assert(rcc->latency_monitor.state == PING_STATE_TIMER);
     red_channel_client_cancel_ping_timer(rcc);
-    /* retrieving the occupied size of the socket's tcp snd buffer (unacked + unsent) */
-    if (ioctl(rcc->stream->socket, TIOCOUTQ, &so_unsent_size) == -1) {
-        spice_printerr("ioctl(TIOCOUTQ) failed, %s", strerror(errno));
-    }
-    if (so_unsent_size > 0) {
-        /* tcp snd buffer is still occupied. rescheduling ping */
-        red_channel_client_start_ping_timer(rcc, PING_TEST_IDLE_NET_TIMEOUT_MS);
-    } else {
-        red_channel_client_push_ping(rcc);
+
+#ifdef HAVE_LINUX_SOCKIOS_H /* SIOCOUTQ is a Linux only ioctl on sockets. */
+    {
+        int so_unsent_size = 0;
+
+        /* retrieving the occupied size of the socket's tcp snd buffer (unacked + unsent) */
+        if (ioctl(rcc->stream->socket, SIOCOUTQ, &so_unsent_size) == -1) {
+            spice_printerr("ioctl(SIOCOUTQ) failed, %s", strerror(errno));
+        }
+        if (so_unsent_size > 0) {
+            /* tcp snd buffer is still occupied. rescheduling ping */
+            red_channel_client_start_ping_timer(rcc, PING_TEST_IDLE_NET_TIMEOUT_MS);
+        } else {
+            red_channel_client_push_ping(rcc);
+        }
     }
+#else /* ifdef HAVE_LINUX_SOCKIOS_H */
+    /* More portable alternative code path (less accurate but avoids bogus ioctls)*/
+    red_channel_client_push_ping(rcc);
+#endif /* ifdef HAVE_LINUX_SOCKIOS_H */
 }
 
 /*
@@ -811,8 +820,8 @@ static void red_channel_client_connectivity_timer(void *opaque)
                                         rcc->connectivity_monitor.timeout);
     } else {
         monitor->state = CONNECTIVITY_STATE_DISCONNECTED;
-        spice_debug("rcc %p has not been responsive for more than %u ms, disconnecting",
-                    rcc, monitor->timeout);
+        spice_warning("rcc %p on channel %d:%d has been unresponsive for more than %u ms, disconnecting",
+                      rcc, rcc->channel->type, rcc->channel->id, monitor->timeout);
         red_channel_client_disconnect(rcc);
     }
 }
@@ -915,7 +924,7 @@ RedChannelClient *red_channel_client_create(int size, RedChannel *channel, RedCl
     red_channel_ref(channel);
     pthread_mutex_unlock(&client->lock);
 
-    if (monitor_latency) {
+    if (monitor_latency && reds_stream_get_family(stream) != AF_UNIX) {
         rcc->latency_monitor.timer = channel->core->timer_add(
             red_channel_client_ping_timer, rcc);
         if (!client->during_target_migrate) {
@@ -1008,7 +1017,7 @@ void red_channel_client_default_migrate(RedChannelClient *rcc)
 }
 
 RedChannel *red_channel_create(int size,
-                               SpiceCoreInterface *core,
+                               const SpiceCoreInterface *core,
                                uint32_t type, uint32_t id,
                                int handle_acks,
                                channel_handle_message_proc handle_message,
@@ -1079,7 +1088,7 @@ static void dummy_watch_remove(SpiceWatch *watch)
 {
 }
 
-// TODO: actually, since I also use channel_client_dummym, no need for core. Can be NULL
+// TODO: actually, since I also use channel_client_dummy, no need for core. Can be NULL
 SpiceCoreInterface dummy_core = {
     .watch_update_mask = dummy_watch_update_mask,
     .watch_add = dummy_watch_add,
@@ -1123,7 +1132,7 @@ static int do_nothing_handle_message(RedChannelClient *rcc,
 }
 
 RedChannel *red_channel_create_parser(int size,
-                               SpiceCoreInterface *core,
+                               const SpiceCoreInterface *core,
                                uint32_t type, uint32_t id,
                                int handle_acks,
                                spice_parse_channel_func_t parser,
@@ -1159,7 +1168,7 @@ void red_channel_register_client_cbs(RedChannel *channel, ClientCbs *client_cbs)
     }
 }
 
-int test_capabilty(const uint32_t *caps, int num_caps, uint32_t cap)
+int test_capability(const uint32_t *caps, int num_caps, uint32_t cap)
 {
     uint32_t index = cap / 32;
     if (num_caps < index + 1) {
@@ -1226,6 +1235,10 @@ static void red_channel_client_unref(RedChannelClient *rcc)
 {
     if (!--rcc->refs) {
         spice_debug("destroy rcc=%p", rcc);
+
+        reds_stream_free(rcc->stream);
+        rcc->stream = NULL;
+
         if (rcc->send_data.main.marshaller) {
             spice_marshaller_destroy(rcc->send_data.main.marshaller);
         }
@@ -1749,7 +1762,7 @@ void red_channel_pipes_add_empty_msg(RedChannel *channel, int msg_type)
 int red_channel_client_is_connected(RedChannelClient *rcc)
 {
     if (!rcc->dummy) {
-        return rcc->stream != NULL;
+        return ring_item_is_linked(&rcc->channel_link);
     } else {
         return rcc->dummy_connected;
     }
@@ -1804,6 +1817,8 @@ static void red_channel_remove_client(RedChannelClient *rcc)
                       rcc->channel->type, rcc->channel->id,
                       rcc->channel->thread_id, pthread_self());
     }
+    spice_return_if_fail(ring_item_is_linked(&rcc->channel_link));
+
     ring_remove(&rcc->channel_link);
     spice_assert(rcc->channel->clients_num > 0);
     rcc->channel->clients_num--;
@@ -1845,8 +1860,6 @@ void red_channel_client_disconnect(RedChannelClient *rcc)
         rcc->channel->core->watch_remove(rcc->stream->watch);
         rcc->stream->watch = NULL;
     }
-    reds_stream_free(rcc->stream);
-    rcc->stream = NULL;
     if (rcc->latency_monitor.timer) {
         rcc->channel->core->timer_remove(rcc->latency_monitor.timer);
         rcc->latency_monitor.timer = NULL;
@@ -2282,7 +2295,7 @@ uint32_t red_channel_max_pipe_size(RedChannel *channel)
 
     RING_FOREACH(link, &channel->clients) {
         rcc = SPICE_CONTAINEROF(link, RedChannelClient, channel_link);
-        pipe_size = pipe_size > rcc->pipe_size ? pipe_size : rcc->pipe_size;
+        pipe_size = MAX(pipe_size, rcc->pipe_size);
     }
     return pipe_size;
 }
@@ -2295,7 +2308,7 @@ uint32_t red_channel_min_pipe_size(RedChannel *channel)
 
     RING_FOREACH(link, &channel->clients) {
         rcc = SPICE_CONTAINEROF(link, RedChannelClient, channel_link);
-        pipe_size = pipe_size < rcc->pipe_size ? pipe_size : rcc->pipe_size;
+        pipe_size = MIN(pipe_size, rcc->pipe_size);
     }
     return pipe_size == ~0 ? 0 : pipe_size;
 }
@@ -2324,6 +2337,8 @@ int red_channel_client_wait_outgoing_item(RedChannelClient *rcc,
     }
     if (timeout != -1) {
         end_time = red_now() + timeout;
+    } else {
+        end_time = UINT64_MAX;
     }
     spice_info("blocked");
 
@@ -2355,6 +2370,8 @@ int red_channel_client_wait_pipe_item_sent(RedChannelClient *rcc,
 
     if (timeout != -1) {
         end_time = red_now() + timeout;
+    } else {
+        end_time = UINT64_MAX;
     }
 
     rcc->channel->channel_cbs.hold_item(rcc, item);
@@ -2392,6 +2409,8 @@ int red_channel_wait_all_sent(RedChannel *channel,
 
     if (timeout != -1) {
         end_time = red_now() + timeout;
+    } else {
+        end_time = UINT64_MAX;
     }
 
     red_channel_push(channel);
