@@ -1,21 +1,29 @@
 /*
-    Copyright (c) 2005-2016 Intel Corporation
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
-    Licensed under the Apache License, Version 2.0 (the "License");
-    you may not use this file except in compliance with the License.
-    You may obtain a copy of the License at
-
-        http://www.apache.org/licenses/LICENSE-2.0
-
-    Unless required by applicable law or agreed to in writing, software
-    distributed under the License is distributed on an "AS IS" BASIS,
-    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-    See the License for the specific language governing permissions and
-    limitations under the License.
+    This file is part of Threading Building Blocks.
 
+    Threading Building Blocks is free software; you can redistribute it
+    and/or modify it under the terms of the GNU General Public License
+    version 2 as published by the Free Software Foundation.
 
+    Threading Building Blocks is distributed in the hope that it will be
+    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
+    You should have received a copy of the GNU General Public License
+    along with Threading Building Blocks; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+    As a special exception, you may use this file as part of a free software
+    library without restriction.  Specifically, if other files instantiate
+    templates or use macros or inline functions from this file, or you compile
+    this file and link it with other files to produce an executable, this
+    file does not by itself cause the resulting executable to be covered by
+    the GNU General Public License.  This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
 */
 
 #ifndef __TBB_task_scheduler_init_H
@@ -81,8 +89,8 @@ public:
         TBB based components run side-by-side or in a nested fashion inside the same
         process.
 
-        The number_of_threads is ignored if any other task_scheduler_inits
-        currently exist.  A thread may construct multiple task_scheduler_inits.
+        The number_of_threads is ignored if any other task_scheduler_inits 
+        currently exist.  A thread may construct multiple task_scheduler_inits.  
         Doing so does no harm because the underlying scheduler is reference counted. */
     void __TBB_EXPORTED_METHOD initialize( int number_of_threads=automatic );
 
@@ -95,18 +103,18 @@ public:
 
     //! Shorthand for default constructor followed by call to initialize(number_of_threads).
 #if __TBB_SUPPORTS_WORKERS_WAITING_IN_TERMINATE
-    task_scheduler_init( int number_of_threads=automatic, stack_size_type thread_stack_size=0, bool wait_workers_in_terminate = false ) : my_scheduler(NULL)
+    task_scheduler_init( int number_of_threads=automatic, stack_size_type thread_stack_size=0, bool wait_workers_in_terminate = false )
 #else
-    task_scheduler_init( int number_of_threads=automatic, stack_size_type thread_stack_size=0 ) : my_scheduler(NULL)
+    task_scheduler_init( int number_of_threads=automatic, stack_size_type thread_stack_size=0 )
 #endif
-    {
+  : my_scheduler(NULL)  {
         // Two lowest order bits of the stack size argument may be taken to communicate
         // default exception propagation mode of the client to be used when the
         // client manually creates tasks in the master thread and does not use
-        // explicit task group context object. This is necessary because newer
-        // TBB binaries with exact propagation enabled by default may be used
+        // explicit task group context object. This is necessary because newer 
+        // TBB binaries with exact propagation enabled by default may be used 
         // by older clients that expect tbb::captured_exception wrapper.
-        // All zeros mean old client - no preference.
+        // All zeros mean old client - no preference. 
         __TBB_ASSERT( !(thread_stack_size & propagation_mode_mask), "Requested stack size is not aligned" );
 #if TBB_USE_EXCEPTIONS
         thread_stack_size |= TBB_USE_CAPTURED_EXCEPTION ? propagation_mode_captured : propagation_mode_exact;
@@ -120,25 +128,25 @@ public:
 
     //! Destroy scheduler for this thread if thread has no other live task_scheduler_inits.
     ~task_scheduler_init() {
-        if( my_scheduler )
+        if( my_scheduler ) 
             terminate();
         internal::poison_pointer( my_scheduler );
     }
     //! Returns the number of threads TBB scheduler would create if initialized by default.
-    /** Result returned by this method does not depend on whether the scheduler
+    /** Result returned by this method does not depend on whether the scheduler 
         has already been initialized.
-
+        
         Because tbb 2.0 does not support blocking tasks yet, you may use this method
-        to boost the number of threads in the tbb's internal pool, if your tasks are
+        to boost the number of threads in the tbb's internal pool, if your tasks are 
         doing I/O operations. The optimal number of additional threads depends on how
         much time your tasks spend in the blocked state.
-
+        
         Before TBB 3.0 U4 this method returned the number of logical CPU in the
         system. Currently on Windows, Linux and FreeBSD it returns the number of
         logical CPUs available to the current process in accordance with its affinity
         mask.
-
-        NOTE: The return value of this method never changes after its first invocation.
+        
+        NOTE: The return value of this method never changes after its first invocation. 
         This means that changes in the process affinity mask that took place after
         this method was first invoked will not affect the number of worker threads
         in the TBB worker threads pool. */
