@@ -1,21 +1,21 @@
 /*
-    Copyright (c) 2005-2017 Intel Corporation
-
-    Licensed under the Apache License, Version 2.0 (the "License");
-    you may not use this file except in compliance with the License.
-    You may obtain a copy of the License at
-
-        http://www.apache.org/licenses/LICENSE-2.0
-
-    Unless required by applicable law or agreed to in writing, software
-    distributed under the License is distributed on an "AS IS" BASIS,
-    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-    See the License for the specific language governing permissions and
-    limitations under the License.
-
-
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef __TBB_atomic_H
@@ -31,8 +31,8 @@
 
 #include "tbb_machine.h"
 
-#if _MSC_VER && !__INTEL_COMPILER
-    // Suppress overzealous compiler warnings till the end of the file
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER)
+    // Workaround for overzealous compiler warnings
     #pragma warning (push)
     #pragma warning (disable: 4244 4267 4512)
 #endif
@@ -54,9 +54,7 @@ enum memory_semantics {
 //! @cond INTERNAL
 namespace internal {
 
-#if __TBB_ALIGNAS_PRESENT
-    #define __TBB_DECL_ATOMIC_FIELD(t,f,a) alignas(a) t f;
-#elif __TBB_ATTRIBUTE_ALIGNED_PRESENT
+#if __TBB_ATTRIBUTE_ALIGNED_PRESENT
     #define __TBB_DECL_ATOMIC_FIELD(t,f,a) t f  __attribute__ ((aligned(a)));
 #elif __TBB_DECLSPEC_ALIGN_PRESENT
     #define __TBB_DECL_ATOMIC_FIELD(t,f,a) __declspec(align(a)) t f;
@@ -553,6 +551,6 @@ inline atomic<T>& as_atomic( T& t ) {
 
 #if _MSC_VER && !__INTEL_COMPILER
     #pragma warning (pop)
-#endif // warnings are restored
+#endif // warnings 4244, 4267 are back
 
 #endif /* __TBB_atomic_H */
